package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.CrossInOutEnum;
import net.wanji.common.enums.StrategyAndMetricsEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.TimeArrayUtil;
import net.wanji.databus.bo.GreenwaveStartEndDateBO;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.databus.vo.CrossIdAndNameVO;
import net.wanji.opt.bo.*;
import net.wanji.opt.common.KafkaConsumerUtil;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.dto.GreenwaveInSectionDTO;
import net.wanji.opt.dto.LineSchemeDTO;
import net.wanji.opt.dto.PhaseEmptyResult;
import net.wanji.opt.po.strategy.SceneStrategyPO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.service.MainlineEvaluateService;
import net.wanji.opt.vo.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.function.Function;
import java.util.function.ToDoubleFunction;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/9/19 15:24
 */
@Slf4j
@Service
public class MainlineEvaluateServiceImpl implements MainlineEvaluateService {
    private final GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    @Value("${spring.kafka.bootstrap-servers}")
    private String bootstrapServers;

    @Value("${spring.kafka.empty-phase-topic}")
    private String emptyPhaseTopic;

    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final BaseCrossTurnInfoMapper baseCrossTurnInfoMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final CrossDataHistMapper crossDataHistMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final GreenwaveInfoMapper greenwaveInfoMapper;
    private final GreenwaveHistMapper greenwaveHistMapper;
    private final GreenwaveCrossMapper greenwaveCrossMapper;
    private final BaseCrossInfoMapper baseCrossInfoMapper;
    private final GreenwaveSceneMapper greenwaveSceneMapper;
    private final SceneStrategyMapper sceneStrategyMapper;
    private final StrategyMapper strategyMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final RidInfoMapper ridInfoMapper;
    private final BaseCrossPlanMapper baseCrossPlanMapper;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final GreenwaveMergeMapper greenwaveMergeMapper;

    SimpleDateFormat hourMinuteFormat = new SimpleDateFormat("HH:mm");
    DateTimeFormatter safeHourMinuteFormat = DateTimeFormatter.ofPattern("HH:mm")
            .withZone(java.time.ZoneId.systemDefault());
    SimpleDateFormat dayFormat = new SimpleDateFormat("yyyy-MM-dd");
    SimpleDateFormat chineseDayFormat = new SimpleDateFormat("M月d日");
    SimpleDateFormat dateHourMinuteFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    // 干线方案缓存 <干线名:干线方案对象>
    public static Map<String, LineSchemeDTO> lineSchemeBuffer = new HashMap<>();

    public MainlineEvaluateServiceImpl(@Qualifier("baseCrossDirInfoMapper") BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                                       @Qualifier("baseCrossTurnInfoMapper") BaseCrossTurnInfoMapper baseCrossTurnInfoMapper,
                                       CrossBaseLaneInfoMapper crossBaseLaneInfoMapper,
                                       @Qualifier("laneInfoMapper") LaneInfoMapper laneInfoMapper,
                                       CrossDirDataHistMapper crossDirDataHistMapper, CrossDataHistMapper crossDataHistMapper, CrossTurnDataHistMapper crossTurnDataHistMapper, CrossLaneDataHistMapper crossLaneDataHistMapper, @Qualifier("greenwaveInfoMapper") GreenwaveInfoMapper greenwaveInfoMapper, @Qualifier("greenwaveHistMapper") GreenwaveHistMapper greenwaveHistMapper, @Qualifier("greenwaveCrossMapper") GreenwaveCrossMapper greenwaveCrossMapper, @Qualifier("baseCrossInfoMapper") BaseCrossInfoMapper baseCrossInfoMapper, @Qualifier("greenwaveSceneMapper") GreenwaveSceneMapper greenwaveSceneMapper, @Qualifier("sceneStrategyMapper") SceneStrategyMapper sceneStrategyMapper, @Qualifier("strategyMapper") StrategyMapper strategyMapper, @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper, @Qualifier("ridInfoMapper") RidInfoMapper ridInfoMapper, @Qualifier("baseCrossPlanMapper") BaseCrossPlanMapper baseCrossPlanMapper, @Qualifier("baseCrossSchemeMapper") BaseCrossSchemeMapper baseCrossSchemeMapper, @Qualifier("greenwaveMergeMapper") GreenwaveMergeMapper greenwaveMergeMapper, @Qualifier("greenwaveRealtimeMapper") GreenwaveRealtimeMapper greenwaveRealtimeMapper) {
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.baseCrossTurnInfoMapper = baseCrossTurnInfoMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.crossDataHistMapper = crossDataHistMapper;
        this.crossTurnDataHistMapper = crossTurnDataHistMapper;
        this.crossLaneDataHistMapper = crossLaneDataHistMapper;
        this.greenwaveInfoMapper = greenwaveInfoMapper;
        this.greenwaveHistMapper = greenwaveHistMapper;
        this.greenwaveCrossMapper = greenwaveCrossMapper;
        this.baseCrossInfoMapper = baseCrossInfoMapper;
        this.greenwaveSceneMapper = greenwaveSceneMapper;
        this.sceneStrategyMapper = sceneStrategyMapper;
        this.strategyMapper = strategyMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.ridInfoMapper = ridInfoMapper;
        this.baseCrossPlanMapper = baseCrossPlanMapper;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.greenwaveMergeMapper = greenwaveMergeMapper;
        this.greenwaveRealtimeMapper = greenwaveRealtimeMapper;
    }

    @Override
    public List<String> bottomMenu(BottomMenuBO bo) {
        String crossId = bo.getCrossId();
        Integer scope = bo.getScope();

        if (scope == 1) {
            return buildDirMenu(crossId);
        } else if (scope == 2) {
            return buildTurnMenu(crossId);
        } else if (scope == 3) {
            return buildLaneMenu(crossId);
        }

        return null;
    }

    @Override
    public List<MainlineEvaluateBottomCurveVO> bottomCurve(BottomCurveBO bo) {
        String crossId = bo.getCrossId();

        // 返回零值
        if (ObjectUtil.isEmpty(crossId)) {
            return new ArrayList<>();
        }

        Integer scope = bo.getScope();
        List<String> scopeList = bo.getScopeList();
        Integer minutes = bo.getMinutes();
        Integer minutesSpan = minutes - 1; // 排除结束时间
        String metricName = bo.getMetricCode();
        Date startTime = bo.getStartTime();
        Date endTime = bo.getEndTime();

        // 从 Kafka 查询绿灯空放时长
        KafkaConsumerUtil kafkaConsumerUtil = new KafkaConsumerUtil(bootstrapServers, "bottom-curve-empty-phase");
        List<PhaseEmptyResult> phaseEmptyResults =
                kafkaConsumerUtil.consumeEmptyPhaseForTimeRange(
                        emptyPhaseTopic, 0, startTime.getTime(), endTime.getTime());
        List<PhaseEmptyResult> crossEmptyPhaseList = phaseEmptyResults.stream()
                .filter(p -> crossId.equals(p.getCrossId()))
                .collect(Collectors.toList());

        String metricCode = StrategyAndMetricsEnum.Metrics.getCodeByName(metricName);
        if (scope != 1 && Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getCode())) {
            throw new RuntimeException("只有路口级别可选择负载均衡度");
        }

        List<String> timeList = buildTimeList(startTime, endTime, minutes);
        List<MainlineEvaluateBottomCurveVO> res = new ArrayList<>();


        int startTimeStamp = (int) (startTime.getTime() / 1000);
        int endTimeStamp = (int) (endTime.getTime() / 1000);

        if (scope == 0) { // 路口
            // 查询路口该时段内的所有数据
            List<CrossDataHistPOExt> crossPOList = crossDataHistMapper.selectExtByCrossIdAndStartEnd(
                    crossId, startTimeStamp, endTimeStamp);

            // 填充相位空放时长
            for (CrossDataHistPOExt po : crossPOList) {
                // 计算结束时间
                Date endTime2 = new Date(po.getStartTime().getTime() + 5 * 60 * 1000);  // 增加5分钟

                // 遍历每个 PhaseEmptyResult 来查找和累加 duration
                for (PhaseEmptyResult phaseEmptyResult : crossEmptyPhaseList) {
                    if (phaseEmptyResult.getDetectTime() * 1000 >= po.getStartTime().getTime()
                            && phaseEmptyResult.getDetectTime() * 1000 <= endTime2.getTime()) {
                        // 累加 duration
                        po.setEmptyPhase(po.getEmptyPhase() + phaseEmptyResult.getDuration());
                    }
                }
            }

            for (String timeStr : timeList) {
                // 解析 timeStr 为 Calendar 对象
                String[] timeParts = timeStr.split(":");
                int hour = Integer.parseInt(timeParts[0]);
                int minute = Integer.parseInt(timeParts[1]);

                Calendar startTimeCal = Calendar.getInstance();
                startTimeCal.set(Calendar.HOUR_OF_DAY, hour);
                startTimeCal.set(Calendar.MINUTE, minute);
                startTimeCal.set(Calendar.SECOND, 0);
                startTimeCal.set(Calendar.MILLISECOND, 0);

                // 计算结束时间
                Calendar endTimeCal = (Calendar) startTimeCal.clone();
                endTimeCal.add(Calendar.MINUTE, minutesSpan);

                // 过滤 crossPOList 中的记录
                List<CrossDataHistPOExt> filteredList = crossPOList.stream()
                        .filter(crossPO -> {
                            // 将 batchTime 转换为小时和分钟
                            Calendar batchTimeCal = Calendar.getInstance();
                            batchTimeCal.setTimeInMillis(crossPO.getBatchTime() * 1000L);  // 将时间戳转换为毫秒
                            String batchTimeStr = hourMinuteFormat.format(batchTimeCal.getTime());

                            // 比较 batchTime 是否在 timeStr 和 endTimeStr 之间
                            String endTimeStr = hourMinuteFormat.format(endTimeCal.getTime());
                            return (batchTimeStr.compareTo(timeStr) >= 0 && batchTimeStr.compareTo(endTimeStr) <= 0);
                        })
                        .collect(Collectors.toList());

                MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                vo.setMetricTime(timeStr);

                if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getNoStopRate)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getOneStopRate)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                    double v = filteredList.stream()
                            .mapToInt(CrossDataHistPOExt::getDelayTime)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getQueueLength)
                            .max()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getStopTimes)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getSpeed)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getGreenLightEfficiency)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getSturation)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getLoadBalance)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getEffusionRate)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.EMPTY_PHASE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getEmptyPhase)
                            .sum();
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.FLOW.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPOExt::getFlow)
                            .sum();
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                }

                res.add(vo);
            }
        } else if (scope == 1) { // 进口道
            // 查询各进口道该时段内的所有数据
            List<CrossDirDataHistPO> dirPOList = crossDirDataHistMapper.selectByCrossIdAndStartEnd(
                    crossId, startTimeStamp, endTimeStamp);

            for (String timeStr : timeList) {
                for (String dirStr : scopeList) {
                    int index = dirStr.indexOf("进口");
                    String dirName = dirStr.substring(0, index);
                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(dirStr);

                    // 解析 timeStr 为 Calendar 对象
                    String[] timeParts = timeStr.split(":");
                    int hour = Integer.parseInt(timeParts[0]);
                    int minute = Integer.parseInt(timeParts[1]);

                    Calendar startTimeCal = Calendar.getInstance();
                    startTimeCal.set(Calendar.HOUR_OF_DAY, hour);
                    startTimeCal.set(Calendar.MINUTE, minute);
                    startTimeCal.set(Calendar.SECOND, 0);
                    startTimeCal.set(Calendar.MILLISECOND, 0);

                    // 计算结束时间
                    Calendar endTimeCal = (Calendar) startTimeCal.clone();
                    endTimeCal.add(Calendar.MINUTE, minutesSpan);

                    // 过滤 poList 中的记录
                    List<CrossDirDataHistPO> filteredList = dirPOList.stream()
                            .filter(po -> po.getDirType().equals(dirCode))
                            .filter(po -> po.getInOutType().equals(1))
                            .filter(po -> {
                                // 将 batchTime 转换为小时和分钟
                                Calendar batchTimeCal = Calendar.getInstance();
                                batchTimeCal.setTimeInMillis(po.getBatchTime() * 1000L);  // 将时间戳转换为毫秒
                                String batchTimeStr = hourMinuteFormat.format(batchTimeCal.getTime());

                                // 比较 batchTime 是否在 timeStr 和 endTimeStr 之间
                                String endTimeStr = hourMinuteFormat.format(endTimeCal.getTime());
                                return (batchTimeStr.compareTo(timeStr) >= 0 && batchTimeStr.compareTo(endTimeStr) <= 0);
                            })
                            .collect(Collectors.toList());
                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossDirDataHistPO::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.FLOW.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getFlow)
                                .sum();
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        } else if (scope == 2) { // 方向
            // 查询各方向该时段内的所有数据
            List<CrossTurnDataHistPO> dirPOList = crossTurnDataHistMapper.selectByCrossId(
                    crossId, endTimeStamp, startTimeStamp);

            for (String timeStr : timeList) {
                for (String turnStr : scopeList) {
                    int index = turnStr.indexOf("进口");
                    String dirName = turnStr.substring(0, index);
                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);
                    String turnName = turnStr.substring(index + 2);
                    String turnCode = TurnConvertEnum.getCodeByDesc(turnName);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(turnStr);

                    // 解析 timeStr 为 Calendar 对象
                    String[] timeParts = timeStr.split(":");
                    int hour = Integer.parseInt(timeParts[0]);
                    int minute = Integer.parseInt(timeParts[1]);

                    Calendar startTimeCal = Calendar.getInstance();
                    startTimeCal.set(Calendar.HOUR_OF_DAY, hour);
                    startTimeCal.set(Calendar.MINUTE, minute);
                    startTimeCal.set(Calendar.SECOND, 0);
                    startTimeCal.set(Calendar.MILLISECOND, 0);

                    // 计算结束时间
                    Calendar endTimeCal = (Calendar) startTimeCal.clone();
                    endTimeCal.add(Calendar.MINUTE, minutesSpan);

                    // 过滤 poList 中的记录
                    List<CrossTurnDataHistPO> filteredList = dirPOList.stream()
                            .filter(po -> po.getInDir().equals(dirCode))
                            .filter(po -> po.getTurnType().equals(turnCode))
                            .filter(po -> {
                                // 将 batchTime 转换为小时和分钟
                                Calendar batchTimeCal = Calendar.getInstance();
                                batchTimeCal.setTimeInMillis(po.getBatchTime() * 1000L);  // 将时间戳转换为毫秒
                                String batchTimeStr = hourMinuteFormat.format(batchTimeCal.getTime());

                                // 比较 batchTime 是否在 timeStr 和 endTimeStr 之间
                                String endTimeStr = hourMinuteFormat.format(endTimeCal.getTime());
                                return (batchTimeStr.compareTo(timeStr) >= 0 && batchTimeStr.compareTo(endTimeStr) <= 0);
                            })
                            .collect(Collectors.toList());
                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossTurnDataHistPO::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.FLOW.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getFlow)
                                .sum();
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        } else if (scope == 3) { // 车道
            // 查询各车道该时段内的所有数据
            List<CrossLaneDataHistPOExt> lanePOList = crossLaneDataHistMapper.selectByCrossId(
                    crossId, endTimeStamp, startTimeStamp);

            for (String timeStr : timeList) {
                for (String laneStr : scopeList) {
                    int indexOfJinKou = laneStr.indexOf("进口");
                    int indexOfCheDao = laneStr.indexOf("车道");
                    // 截取"进口"之前的部分
                    String dirName = laneStr.substring(0, indexOfJinKou);
                    // 截取"车道"之前的数字部分
                    String laneSortStr = laneStr.substring(indexOfCheDao - 2, indexOfCheDao);

                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);
                    Integer laneSort = Integer.parseInt(laneSortStr);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(laneStr);

                    // 解析 timeStr 为 Calendar 对象
                    String[] timeParts = timeStr.split(":");
                    int hour = Integer.parseInt(timeParts[0]);
                    int minute = Integer.parseInt(timeParts[1]);

                    Calendar startTimeCal = Calendar.getInstance();
                    startTimeCal.set(Calendar.HOUR_OF_DAY, hour);
                    startTimeCal.set(Calendar.MINUTE, minute);
                    startTimeCal.set(Calendar.SECOND, 0);
                    startTimeCal.set(Calendar.MILLISECOND, 0);

                    // 计算结束时间
                    Calendar endTimeCal = (Calendar) startTimeCal.clone();
                    endTimeCal.add(Calendar.MINUTE, minutesSpan);

                    // 过滤 poList 中的记录
                    List<CrossLaneDataHistPOExt> filteredList = lanePOList.stream()
                            .filter(po -> po.getDir().equals(dirCode))
                            .filter(po -> po.getSort().equals(laneSort))
                            .filter(po -> {
                                // 将 batchTime 转换为小时和分钟
                                Calendar batchTimeCal = Calendar.getInstance();
                                batchTimeCal.setTimeInMillis(po.getBatchTime() * 1000L);  // 将时间戳转换为毫秒
                                String batchTimeStr = hourMinuteFormat.format(batchTimeCal.getTime());

                                // 比较 batchTime 是否在 timeStr 和 endTimeStr 之间
                                String endTimeStr = hourMinuteFormat.format(endTimeCal.getTime());
                                return (batchTimeStr.compareTo(timeStr) >= 0 && batchTimeStr.compareTo(endTimeStr) <= 0);
                            })
                            .collect(Collectors.toList());

                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossLaneDataHistPOExt::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.FLOW.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getFlow)
                                .sum();
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        }

        res.sort(Comparator.comparing(MainlineEvaluateBottomCurveVO::getMetricTime));
        return res;
    }

    @Override
    public List<MainlineListVO> mainlineList() {
        List<MainlineListVO> res = new ArrayList<>();

        // 按干线名称过滤数据
        List<GreenwaveInfoPO> poList = greenwaveInfoMapper.selectAll();
        List<GreenwaveInfoPO> filteredList = poList.stream()
                .collect(Collectors.groupingBy(po -> getPrefix(po.getName())))
                .values()
                .stream()
                .filter(list -> !list.isEmpty())
                .map(list -> list.get(0))
                .collect(Collectors.toList());

        for (GreenwaveInfoPO greenwaveInfoPO : filteredList) {
            MainlineListVO mainlineListVO = new MainlineListVO();

            String name = greenwaveInfoPO.getName(); // 例：旅游路-转山西路至霞景路路段:双向绿波1
            String prefix = getPrefix(name);
            mainlineListVO.setMainlineName(prefix);

            Integer greenwaveId = greenwaveInfoPO.getId();
            GreenwaveRealtimePO greenwaveRealtimePO = greenwaveRealtimeMapper.selectById(greenwaveId);

            if (ObjectUtil.isNotEmpty(greenwaveRealtimePO)) {
                Double trafficIndex = greenwaveRealtimePO.getTrafficIndex();
                double roundedTrafficIndex = Math.round(trafficIndex * 100.0) / 100.0;
                mainlineListVO.setTrafficIndex(roundedTrafficIndex);
                mainlineListVO.setGreenId(greenwaveRealtimePO.getGreenId());

            }
            setCrossListAndwkt(mainlineListVO, greenwaveId);
            res.add(mainlineListVO);
        }

        res.sort(Comparator.comparingDouble(MainlineListVO::getTrafficIndex).reversed());
        return res;
    }

    private void setCrossListAndwkt(MainlineListVO mainlineListVO, Integer greenwaveId) {
        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        Map<String, GreenwaveCrossPO> uniqueCrossIdMap = greenwaveCrossPOList.stream()
                .collect(Collectors.toMap(
                        GreenwaveCrossPO::getCrossId,
                        Function.identity(),
                        (existing, replacement) -> existing
                ));

        List<GreenwaveCrossPO> filteredList = new ArrayList<>(uniqueCrossIdMap.values());
        filteredList.sort(Comparator.comparing(GreenwaveCrossPO::getSort));


        List<CrossIdAndNameVO> crossIdAndNameVOList = new ArrayList<>();
        StringBuilder wkt = new StringBuilder();

        for (int i = 0; i < filteredList.size() - 1; i++) {
            GreenwaveCrossPO preCross = filteredList.get(i);
            GreenwaveCrossPO nextCross = filteredList.get(i + 1);

            // 构造路口列表
            String crossId = preCross.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
            crossIdAndNameVO.setCrossId(crossId);
            crossIdAndNameVO.setCrossName(baseCrossInfoPO.getName());
            crossIdAndNameVOList.add(crossIdAndNameVO);
            if (i == filteredList.size() - 2) {
                String crossId1 = nextCross.getCrossId();
                BaseCrossInfoPO baseCrossInfoPO1 = baseCrossInfoMapper.selectById(crossId1);
                CrossIdAndNameVO crossIdAndNameVO1 = new CrossIdAndNameVO();
                crossIdAndNameVO1.setCrossId(crossId1);
                crossIdAndNameVO1.setCrossName(baseCrossInfoPO1.getName());
                crossIdAndNameVOList.add(crossIdAndNameVO1);
            }

            // 构造wkt
            RidInfoEntity ridInfo = ridInfoMapper.selectByStartEnd(preCross.getCrossId(), nextCross.getCrossId());
            if (i == 0) {
                wkt.append(ridInfo.getWkt());
            } else {
                wkt.append(";").append(ridInfo.getWkt());
            }
        }

        mainlineListVO.setCrossList(crossIdAndNameVOList);
        mainlineListVO.setWkt(wkt.toString());
    }


    private String getPrefix(String name) {
        int index = name.indexOf(':');
        return index != -1 ? name.substring(0, index).trim() : name;
    }

    @Override
    public MainlineSchemeAnalysisVO mainlineSchemeAnalysis(MainlineSchemeAnalysisBO bo) throws Exception {
        MainlineSchemeAnalysisVO res = null;
        try {
            lineSchemeBuffer.clear();
            String mainlineName = bo.getName();
            Date boStartTime = bo.getStartTime();
            Date boEndTime = bo.getEndTime();
            String boStartTimeStr = sdf.format(boStartTime);
            String boEndTimeStr = sdf.format(boEndTime);
            int boStartTimeStamp = (int) (boStartTime.getTime() / 1000);
            int boEndTimeStamp = (int) (boEndTime.getTime() / 1000);
            // 根据时段查询所有发生的绿波
            List<GreenwaveHistPO> greenwaveHistPOList = greenwaveHistMapper
                    .selectByTimeSection(boStartTimeStr, boEndTimeStr);

            // 根据干线名称筛选结果
            List<GreenwaveHistPO> filteredList = greenwaveHistPOList.stream()
                    .filter(po -> {
                        String name = po.getGreenwaveName();
                        int index = name.indexOf(':');
                        if (index != -1) {
                            String leftSide = name.substring(0, index).trim();
                            return leftSide.equals(mainlineName);
                        }
                        return false;
                    })
                    .collect(Collectors.toList());

            res = new MainlineSchemeAnalysisVO();
            fillLineSchemeBuffer(filteredList, lineSchemeBuffer);
            // 时段合并
            GreenwaveMergePO greenwaveMergePO = greenwaveMergeMapper.selectByMainlineName(mainlineName);
            if (ObjectUtil.isNotEmpty(greenwaveMergePO)) {
                mergeLineSchemeBuffer(lineSchemeBuffer, mainlineName, greenwaveMergePO);
            }
            res.setSceneData(calcSceneData(lineSchemeBuffer));
            res.setEvaluateData(calcEvaluateData(
                    filteredList, lineSchemeBuffer, boStartTimeStamp, boEndTimeStamp, boStartTime, boEndTime));
            res.setGreenwaveData(buildGreenwaveData(lineSchemeBuffer));
            res.setCrossData(calcCrossData(lineSchemeBuffer));
        } catch (Exception e) {
            log.error("干线方案分析执行异常：", e);
            throw new Exception(e);
        }

        return res;
    }

    private void fillLineSchemeBuffer(List<GreenwaveHistPO> filteredList, Map<String, LineSchemeDTO> lineSchemeBuffer)
            throws Exception {

        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectAll();
        List<CrossSectionPO> baseCrossSectionPOList = baseCrossSectionMapper.selectAll();
        List<BaseCrossPlanPO> baseCrossPlanPOList = baseCrossPlanMapper.selectAll();

        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Integer histGreenwaveId = greenwaveHistPO.getGreenId();
            Date histGreenwaveGmtModified = greenwaveHistPO.getGmtModified();

            for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOList) {
                Integer sectionId = greenwaveCrossPO.getSectionId();
                // 判断绿波历史发生时间是否在时段内
                GreenwaveInSectionDTO greenwaveInSectionDTO = isGreenwaveInSection(
                        histGreenwaveGmtModified, sectionId, baseCrossSectionPOList, baseCrossPlanPOList);
                if (greenwaveInSectionDTO.isGreenwaveInSection()) {
                    String greenwaveName = greenwaveHistPO.getGreenwaveName();
                    Integer greenwaveId = greenwaveHistPO.getGreenId();
                    String planName = greenwaveInSectionDTO.getPlanName();
                    Integer planId = greenwaveInSectionDTO.getPlanId();

                    String startTime = greenwaveInSectionDTO.getStartTime();
                    String endTime = greenwaveInSectionDTO.getEndTime();

                    String lineSchemeName = greenwaveName + " " + planName + " " + startTime + "-" + endTime;
                    LineSchemeDTO lineSchemeDTO = lineSchemeBuffer.getOrDefault(lineSchemeName, new LineSchemeDTO());
                    List<GreenwaveHistPO> greenwaveHistPOList = lineSchemeDTO.getGreenwaveHistPOList();
                    greenwaveHistPOList.add(greenwaveHistPO);
                    lineSchemeDTO.setSectionId(sectionId);
                    lineSchemeDTO.setPlanId(planId);
                    lineSchemeDTO.setGreenwaveId(greenwaveId);
                    lineSchemeBuffer.put(lineSchemeName, lineSchemeDTO);
                }
            }
        }
    }

    private void mergeLineSchemeBuffer(Map<String, LineSchemeDTO> lineSchemeBuffer,
                                       String mainlineName, GreenwaveMergePO greenwaveMergePO) {

        Map<String, LineSchemeDTO> mergedRecords = new HashMap<>();

        Iterator<String> iterator = lineSchemeBuffer.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            String[] keyParts = key.split(" ");
            String timeRange = keyParts[keyParts.length - 1];
            String[] times = timeRange.split("-");
            int startTime = timeToInt(times[0]);
            int endTime = timeToInt(times[1]);

            boolean isWeekday = key.contains("工作日");
            List<int[]> mergingPeriods = getMergingPeriods(isWeekday, greenwaveMergePO);

            for (int[] mergingPeriod : mergingPeriods) {
                if (isOverlapping(startTime, endTime, mergingPeriod[0], mergingPeriod[1])) {
                    int mergedStartTime = mergingPeriod[0];
                    int mergedEndTime = mergingPeriod[1];

                    String newKey = constructNewKey(key, intToTime(mergedStartTime), intToTime(mergedEndTime));
                    if (mergedRecords.containsKey(newKey)) {
                        LineSchemeDTO existingDto = mergedRecords.get(newKey);
                        existingDto.getGreenwaveHistPOList().addAll(lineSchemeBuffer.get(key).getGreenwaveHistPOList());
                        iterator.remove();
                    } else {
                        mergedRecords.put(newKey, lineSchemeBuffer.get(key));
                        iterator.remove();
                    }
                }
            }
        }
        lineSchemeBuffer.putAll(mergedRecords);
    }

    private int timeToInt(String time) {
        String[] parts = time.split(":");
        return Integer.parseInt(parts[0]) * 60 + Integer.parseInt(parts[1]);
    }

    private List<int[]> getMergingPeriods(boolean isWeekday, GreenwaveMergePO greenwaveMergePO) {
        List<int[]> res = new ArrayList<>();
        if (isWeekday) {
            String weekdayMerge = greenwaveMergePO.getWeekdayMerge(); // 07:00,09:30|09:30,16:30|16:30,19:00|21:30,23:59
            String[] split = weekdayMerge.split("\\|");
            List<String> weekDays = new ArrayList<>(Arrays.asList(split));
            fillRes(weekDays, res);
        } else {
            String weekendMerge = greenwaveMergePO.getWeekendMerge(); // 13:30,20:00|21:30,23:59
            String[] split = weekendMerge.split("\\|");
            List<String> weekEnds = new ArrayList<>(Arrays.asList(split));

            fillRes(weekEnds, res);
        }
        return res;
    }

    private void fillRes(List<String> days, List<int[]> res) {
        for (String day : days) {
            // "07:00,09:30"
            String[] split = day.split(",");
            int[] ints = {timeToInt(split[0]), timeToInt(split[1])};
            res.add(ints);
        }
    }

    private boolean isOverlapping(int start1, int end1, int start2, int end2) {
        return (start1 >= start2 && end1 <= end2);
    }

    private static String constructNewKey(String oldKey, String newStartTime, String newEndTime) {
        String[] keyParts = oldKey.split(" ");
        keyParts[keyParts.length - 1] = newStartTime + "-" + newEndTime;
        return String.join(" ", keyParts);
    }

    private static String intToTime(int time) {
        return String.format("%02d:%02d", time / 60, time % 60);
    }

    @Override
    public MainlineSchemeEvaluateVO mainlineSchemeEvaluate(MainlineSchemeEvaluateBO bo) {
        String greenwaveName = bo.getGreenwaveName();

        if (ObjectUtil.isEmpty(greenwaveName)) {
            return emptyResMainlineSchemeEvaluate(bo);
        }

        LineSchemeDTO lineSchemeDTO = lineSchemeBuffer.get(greenwaveName);
        Integer greenwaveId = lineSchemeDTO.getGreenwaveId();
        String dirName = bo.getDirName();
        String metricName = bo.getMetricName();
        Date boStartTime = bo.getStartTime();
        Date boEndTime = bo.getEndTime();
        Integer isCityBrain = bo.getIsCityBrain();

        MainlineSchemeEvaluateVO res = new MainlineSchemeEvaluateVO();

        String[] split = greenwaveName.split(" "); // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 07:00-09:00
        String s = split[split.length - 1];
        String[] split1 = s.split("-");
        String startHourMinuteStr = split1[0];
        String endHourMinuteStr = split1[1];
        res.setTimeSection(startHourMinuteStr + "-" + endHourMinuteStr);

        // 确定控制策略
        StrategyPO strategyPO = getStrategyPO(greenwaveId);
        String strategyName = strategyPO.getStrategyName();
        res.setStrategyName(strategyName);

        res.setGreenwaveCrossList(buildGreenwaveCrossList(greenwaveId));

        String strategyCode = strategyPO.getStrategyCode();
        setCurveElementListAndEvaluateList(dirName, metricName, greenwaveId, boStartTime, boEndTime, startHourMinuteStr,
                endHourMinuteStr, res, strategyCode, isCityBrain);

        return res;
    }

    private MainlineSchemeEvaluateVO emptyResMainlineSchemeEvaluate(MainlineSchemeEvaluateBO bo) {
        MainlineSchemeEvaluateVO res = new MainlineSchemeEvaluateVO();
        res.setTimeSection("00:00-05:00");
        res.setStrategyName("干线效率提升");
        res.setGreenwaveCrossList(new ArrayList<>());

        List<MainlineSchemeEvaluateVO.CurveElement> curveElementList = new ArrayList<>();
        List<String> timeIntervals = TimeArrayUtil.getCustomTimeIntervals("00:00", "05:00", 5);
        for (String hourMinute : timeIntervals) {
            MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
            curveElement.setMetricTime(hourMinute);
            curveElement.setValue((double) 0);
            curveElementList.add(curveElement);
        }
        res.setCurveElementList(curveElementList);

        List<String> evaluateList = new ArrayList<>();
        evaluateList.add("");
        res.setEvaluateList(evaluateList);

        return res;
    }

    @Override
    public List<MainlineCrossEvaluateVO> mainlineCrossEvaluate(MainlineCrossEvaluateBO bo) {
        String greenwaveName = bo.getGreenwaveName();

        // 返回零值
        if (ObjectUtil.isEmpty(greenwaveName)) {
            List<MainlineCrossEvaluateVO> res = new ArrayList<>();
            MainlineCrossEvaluateVO vo = new MainlineCrossEvaluateVO();
            vo.setCrossName("");
            res.add(vo);
            return res;
        }

        String metricName = bo.getMetricName();
        List<String> crossIdList = bo.getCrossIdList();
        Date startDate = bo.getStartTime();
        Date endDate = bo.getEndTime();
        String startHourMinute = bo.getStartHourMinute();
        String endHourMinute = bo.getEndHourMinute();

        List<MainlineCrossEvaluateVO> res = new ArrayList<>();

        List<BaseCrossInfoPO> baseCrossInfoPOList = baseCrossInfoMapper.selectAll();
        List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();

        String boStartDayStr = dayFormat.format(startDate);
        String boEndDayStr = dayFormat.format(endDate);
        List<CrossDirDataHistPOExt> crossDirDataHistPOList = crossDirDataHistMapper.selectExtByTimeSection(
                boStartDayStr, boEndDayStr, startHourMinute, endHourMinute);

        List<BaseCrossDirInfoPO> baseCrossDirInfoPOList =
                baseCrossDirInfoMapper.selectByInOutType(CrossInOutEnum.IN.getCode());

        // 从 Kafka 获取绿灯空放数据
        KafkaConsumerUtil kafkaConsumerUtil =
                new KafkaConsumerUtil(bootstrapServers, "mainline-empty-phase-comsumer");
        List<PhaseEmptyResult> phaseEmptyResults =
                kafkaConsumerUtil.consumeEmptyPhaseForTimeRange(
                        emptyPhaseTopic, 0, startDate.getTime(), endDate.getTime());

        for (String crossId : crossIdList) {
            List<PhaseEmptyResult> crossEmptyPhaseList = phaseEmptyResults.stream().
                    filter(p -> crossId.equals(p.getCrossId()))
                    .collect(Collectors.toList());

            MainlineCrossEvaluateVO mainlineCrossEvaluateVO = new MainlineCrossEvaluateVO();

            Optional<BaseCrossInfoPO> result = baseCrossInfoPOList.stream()
                    .filter(item -> crossId.equals(item.getId()))
                    .findFirst();
            BaseCrossInfoPO baseCrossInfoPO = result.get();

            String crossName = baseCrossInfoPO.getName();
            mainlineCrossEvaluateVO.setCrossName(crossName);

            // 确定协调方向
            String[] split = greenwaveName.split(" ");
            String dirStr = split[1];
            String[] split1 = dirStr.split("向");
            String lineName = split[0] + " " + split[1] + " " + split[2];

            Optional<GreenwaveInfoPO> result1 = greenwaveInfoPOList.stream()
                    .filter(item -> lineName.equals(item.getName()))
                    .findFirst();
            GreenwaveInfoPO greenwaveInfoPO = result1.get();
            Integer dir = greenwaveInfoPO.getDir();

            List<Integer> coordDirCodeList = new ArrayList<>();
            if (dir == 0) { // 正向
                String dirStr1 = split1[0];
                Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
                dirCode = convertDirCode(dirStr1, crossId, dirCode);
                coordDirCodeList.add(dirCode);
            } else if (dir == 1) { // 反向
                String dirStr1 = split1[1];
                Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
                dirCode = convertDirCode(dirStr1, crossId, dirCode);
                coordDirCodeList.add(dirCode);
            } else { // 双向
                String dirStr1 = split1[0];
                Integer dirCode1 = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
                dirCode1 = convertDirCode(dirStr1, crossId, dirCode1);
                coordDirCodeList.add(dirCode1);
                String dirStr2 = split1[1];
                Integer dirCode2 = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr2);
                dirCode2 = convertDirCode(dirStr2, crossId, dirCode2);
                coordDirCodeList.add(dirCode2);
            }
            mainlineCrossEvaluateVO.setCoordValue(calcCoordValue(coordDirCodeList, crossId, crossDirDataHistPOList));
            fillNonCoordValue(mainlineCrossEvaluateVO, coordDirCodeList, crossId, crossDirDataHistPOList,
                    baseCrossDirInfoPOList);
            mainlineCrossEvaluateVO.setDirElementList(buildDirElementList(
                    crossId, metricName, baseCrossDirInfoPOList, crossDirDataHistPOList, crossEmptyPhaseList));
            mainlineCrossEvaluateVO.setCrossEvaluateList(buildCrossEvaluateList(
                    crossId, baseCrossDirInfoPOList, crossDirDataHistPOList));
            res.add(mainlineCrossEvaluateVO);
        }
        return res;
    }

    @Override
    public List<MainlineRunningAnalyseVO> mainlineRunningAnalyse(GreenwaveStartEndDateBO bo) {
        Integer greenwaveId = bo.getId();

        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();
        if (isZeros(endDate)) {
            endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        }
        long startTime = startDate.getTime();
        long endTime = endDate.getTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String boStartTimeStr = sdf.format(startDate);
        String boEndTimeStr = sdf.format(endDate);

        List<GreenwaveHistPO> greenwaveHistPOList =
                greenwaveHistMapper.selectByIdAndTimeSection(greenwaveId, boStartTimeStr, boEndTimeStr);

        List<MainlineRunningAnalyseVO> congestionList = new ArrayList<>();
        // 返回零值
        if (CollectionUtil.isEmpty(greenwaveHistPOList)) {
            MainlineRunningAnalyseVO vo = new MainlineRunningAnalyseVO();
            vo.setStatus(3);
            vo.setTimeSpan("");
            vo.setDuration(0);
            vo.setStartTime(null);
            vo.setEndTime(null);
            congestionList.add(vo);
            MainlineRunningAnalyseVO vo2 = new MainlineRunningAnalyseVO();
            vo2.setStatus(2);
            vo2.setTimeSpan("");
            vo2.setDuration(0);
            vo2.setStartTime(null);
            vo2.setEndTime(null);
            congestionList.add(vo2);
            return congestionList;
        }

        // 找出拥堵时段
        for (GreenwaveHistPO po : greenwaveHistPOList) {
            if (po.getStatus() == 3) {
                MainlineRunningAnalyseVO vo = new MainlineRunningAnalyseVO();
                vo.setStartTime(po.getStartTime());
                vo.setEndTime(po.getGmtModified());
                congestionList.add(vo);
            }
        }

        // 找出溢出时段
        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        // 找出绿波包含的路口
        Map<String, GreenwaveCrossPO> uniqueMap = greenwaveCrossPOList.stream()
                .collect(Collectors.toMap(
                        GreenwaveCrossPO::getCrossId,    // 使用crossId作为键
                        Function.identity(),              // 使用对象本身作为值
                        (existing, replacement) -> existing));  // 如果有重复的key, 保留第一个
        List<GreenwaveCrossPO> uniqueList = new ArrayList<>(uniqueMap.values());
        // 路口ID对应的路口历史数据
        Map<String, List<CrossDataHistPO>> crossIdDataMap = new HashMap<>();
        for (GreenwaveCrossPO greenwaveCrossPO : uniqueList) {
            String crossId = greenwaveCrossPO.getCrossId();
            List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                    crossId, (int) (startTime / 1000), (int) (endTime / 1000));
            crossIdDataMap.put(crossId, crossDataHistPOList);
        }
        List<MainlineRunningAnalyseVO> spilloverList = crossIdDataMap.values().stream()
                .flatMap(List::stream)
                .filter(hist -> hist.getStatus() == 3)
                .map(hist -> new MainlineRunningAnalyseVO(2, hist.getStartTime(), hist.getEndTime()))
                .collect(Collectors.toList());
        // 合并交叉时段
        List<MainlineRunningAnalyseVO> finalSpilloverList = mergeIntervals(spilloverList);

        // 用溢出覆盖拥堵
        List<MainlineRunningAnalyseVO> congestionSpilloverList = mergeAndAdjustLists(congestionList, finalSpilloverList);

        SimpleDateFormat sdf2 = new SimpleDateFormat("H:mm");
        Iterator<MainlineRunningAnalyseVO> iterator = congestionSpilloverList.iterator();
        while (iterator.hasNext()) {
            MainlineRunningAnalyseVO item = iterator.next();
            if (item.getStartTime().equals(item.getEndTime())) {
                iterator.remove();
            } else {
                long durationInSeconds = (item.getEndTime().getTime() - item.getStartTime().getTime()) / 1000;
                String timeSpan = sdf2.format(item.getStartTime()) + "-" + sdf2.format(item.getEndTime());
                item.setDuration((int) durationInSeconds);
                item.setTimeSpan(timeSpan);
            }
        }

        return congestionSpilloverList;
    }

    private List<MainlineRunningAnalyseVO> mergeAndAdjustLists(List<MainlineRunningAnalyseVO> congestionList,
                                                               List<MainlineRunningAnalyseVO> finalSpilloverList) {
        List<MainlineRunningAnalyseVO> congestionSpilloverList = new ArrayList<>();

        for (MainlineRunningAnalyseVO item : congestionList) {
            MainlineRunningAnalyseVO newItem = new MainlineRunningAnalyseVO(3, item.getStartTime(), item.getEndTime());
            congestionSpilloverList.add(newItem);
        }

        for (MainlineRunningAnalyseVO spilloverItem : finalSpilloverList) {
            for (MainlineRunningAnalyseVO congestionItem : congestionSpilloverList) {
                if (congestionItem.getStatus() == 3 && overlaps(congestionItem, spilloverItem)) {
                    // 以溢出时段为准修改拥堵时段
                    adjustTimes(congestionItem, spilloverItem);
                }
            }
            congestionSpilloverList.add(new MainlineRunningAnalyseVO(
                    2, spilloverItem.getStartTime(), spilloverItem.getEndTime()));
        }

        return congestionSpilloverList;
    }

    private void adjustTimes(MainlineRunningAnalyseVO congestionItem, MainlineRunningAnalyseVO spilloverItem) {
        if (congestionItem.getStartTime().before(spilloverItem.getStartTime())) {
            congestionItem.setEndTime(spilloverItem.getStartTime());
        } else if (congestionItem.getEndTime().after(spilloverItem.getEndTime())) {
            congestionItem.setStartTime(spilloverItem.getEndTime());
        } else {
            congestionItem.setEndTime(congestionItem.getStartTime()); // 待删除
        }
    }

    private boolean overlaps(MainlineRunningAnalyseVO congestionItem, MainlineRunningAnalyseVO spilloverItem) {
        return congestionItem.getEndTime().after(spilloverItem.getStartTime())
                && congestionItem.getStartTime().before(spilloverItem.getEndTime());

    }

    private List<MainlineRunningAnalyseVO> mergeIntervals(List<MainlineRunningAnalyseVO> intervals) {
        if (intervals.size() <= 1) {
            return intervals;
        }
        // 按开始时间排序
        intervals.sort(Comparator.comparing(MainlineRunningAnalyseVO::getStartTime));

        List<MainlineRunningAnalyseVO> result = new ArrayList<>();
        MainlineRunningAnalyseVO current = intervals.get(0);

        for (MainlineRunningAnalyseVO interval : intervals) {
            if (interval.getStartTime().before(current.getEndTime()) || interval.getStartTime().equals(current.getEndTime())) {
                // 合并交叉时段
                current.setEndTime(new Date(Math.max(current.getEndTime().getTime(), interval.getEndTime().getTime())));
            } else {
                // 未出现交叉
                result.add(current);
                current = interval;
            }
        }
        result.add(current);
        return result;
    }

    private boolean isZeros(Date endDate) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(endDate);
        return calendar.get(Calendar.HOUR_OF_DAY) == 0 &&
                calendar.get(Calendar.MINUTE) == 0 &&
                calendar.get(Calendar.SECOND) == 0 &&
                calendar.get(Calendar.MILLISECOND) == 0;
    }

    private void fillNonCoordValue(MainlineCrossEvaluateVO mainlineCrossEvaluateVO, List<Integer> coordDirCodeList,
                                   String crossId, List<CrossDirDataHistPOExt> crossDirDataHistPOList,
                                   List<BaseCrossDirInfoPO> baseCrossDirInfoPOList) {

        // 筛选非协调方向
        List<BaseCrossDirInfoPO> collect = baseCrossDirInfoPOList.stream()
                .filter(item -> crossId.equals(item.getCrossId()))
                .collect(Collectors.toList());
        List<Integer> nonCoordDirCodeList = collect.stream()
                .map(BaseCrossDirInfoPO::getDirType)
                .collect(Collectors.toList());
        nonCoordDirCodeList.removeAll(coordDirCodeList);

        // 查询时段内路口非协调方向数据
        List<CrossDirDataHistPOExt> filteredList = crossDirDataHistPOList.stream()
                .filter(item -> crossId.equals(item.getCrossId()))
                .filter(item -> 1 == item.getInOutType())
                .filter(item -> nonCoordDirCodeList.contains(item.getDirType()))
                .collect(Collectors.toList());

        // 计算非协调方向指标(排队长度)
        double nonCoordValueDouble = (filteredList.stream()
                .mapToDouble(CrossDirDataHistPOExt::getQueueLength)
                .max()
                .orElse(0.0));
        int nonCoordValue = (int) Math.round(nonCoordValueDouble);
        mainlineCrossEvaluateVO.setNonCoordValue(nonCoordValue);
        mainlineCrossEvaluateVO.setQueueList(buildQueueList(filteredList));

        // 计算非协调停车次数
        double stopTimes = (filteredList.stream()
                .mapToDouble(CrossDirDataHistPOExt::getStopTimes)
                .max()
                .orElse(0.0));
        BigDecimal bd = new BigDecimal(stopTimes);
        bd = bd.setScale(2, RoundingMode.HALF_UP);
        double formattedDouble = bd.doubleValue();
        mainlineCrossEvaluateVO.setStopTimes(formattedDouble);
        mainlineCrossEvaluateVO.setStopTimesList(buildStopList(filteredList));

        // 计算非协调二次停车率
        double twoStopRate = (filteredList.stream()
                .mapToDouble(CrossDirDataHistPOExt::getTwoStopRate)
                .max()
                .orElse(0.0));
        int intRate = (int) (twoStopRate * 100);
        mainlineCrossEvaluateVO.setTwoStopRate(intRate);
        mainlineCrossEvaluateVO.setTwoStopRateList(buildTwoStopRateList(filteredList));
    }

    private List<MainlineCrossEvaluateVO.TimeAndValue> buildQueueList(List<CrossDirDataHistPOExt> filteredList) {
        List<MainlineCrossEvaluateVO.TimeAndValue> result = new ArrayList<>();
        Map<String, List<Double>> groupedByTime = filteredList.stream()
                .collect(Collectors.groupingBy(
                        item -> LocalDateTime.ofEpochSecond(item.getBatchTime(), 0, ZoneOffset.ofHours(8))
                                .format(safeHourMinuteFormat),
                        Collectors.mapping(CrossDirDataHistPOExt::getQueueLength, Collectors.toList())
                ));
        for (Map.Entry<String, List<Double>> entry : groupedByTime.entrySet()) {
            double average = entry.getValue().stream()
                    .mapToDouble(Double::doubleValue)
                    .average()
                    .orElse(0.0);
            result.add(new MainlineCrossEvaluateVO.TimeAndValue(entry.getKey(), (int) Math.round(average)));
        }
        result.sort(Comparator.comparing(MainlineCrossEvaluateVO.TimeAndValue::getTime));
        return result;
    }

    private List<MainlineCrossEvaluateVO.TimeAndValue> buildStopList(List<CrossDirDataHistPOExt> filteredList) {
        List<MainlineCrossEvaluateVO.TimeAndValue> result = new ArrayList<>();
        Map<String, List<Double>> groupedByTime = filteredList.stream()
                .collect(Collectors.groupingBy(
                        item -> LocalDateTime.ofEpochSecond(item.getBatchTime(), 0, ZoneOffset.ofHours(8))
                                .format(safeHourMinuteFormat),
                        Collectors.mapping(CrossDirDataHistPOExt::getStopTimes, Collectors.toList())
                ));
        for (Map.Entry<String, List<Double>> entry : groupedByTime.entrySet()) {
            double average = entry.getValue().stream()
                    .mapToDouble(Double::doubleValue)
                    .average()
                    .orElse(0.0);
            double v = BigDecimal.valueOf(average).setScale(2, RoundingMode.HALF_UP).doubleValue();
            result.add(new MainlineCrossEvaluateVO.TimeAndValue(entry.getKey(), v));
        }
        result.sort(Comparator.comparing(MainlineCrossEvaluateVO.TimeAndValue::getTime));
        return result;
    }

    private List<MainlineCrossEvaluateVO.TimeAndValue> buildTwoStopRateList(List<CrossDirDataHistPOExt> filteredList) {
        List<MainlineCrossEvaluateVO.TimeAndValue> result = new ArrayList<>();
        Map<String, List<Double>> groupedByTime = filteredList.stream()
                .collect(Collectors.groupingBy(
                        item -> LocalDateTime.ofEpochSecond(item.getBatchTime(), 0, ZoneOffset.ofHours(8))
                                .format(safeHourMinuteFormat),
                        Collectors.mapping(CrossDirDataHistPOExt::getTwoStopRate, Collectors.toList())
                ));
        for (Map.Entry<String, List<Double>> entry : groupedByTime.entrySet()) {
            double average = entry.getValue().stream()
                    .mapToDouble(Double::doubleValue)
                    .average()
                    .orElse(0.0);
            result.add(new MainlineCrossEvaluateVO.TimeAndValue(entry.getKey(), (int)(average * 100)));
        }
        result.sort(Comparator.comparing(MainlineCrossEvaluateVO.TimeAndValue::getTime));
        return result;
    }

    private List<String> buildCrossEvaluateList(
            String crossId, List<BaseCrossDirInfoPO> baseCrossDirInfoPOList,
            List<CrossDirDataHistPOExt> crossDirDataHistPOList) {

        List<String> res = new ArrayList<>();

        // 获取路口所有方向
        List<BaseCrossDirInfoPO> collect = baseCrossDirInfoPOList.stream()
                .filter(item -> crossId.equals(item.getCrossId()))
                .collect(Collectors.toList());
        List<Integer> dirCodeList = collect.stream()
                .map(BaseCrossDirInfoPO::getDirType)
                .collect(Collectors.toList());

        for (Integer dirCode : dirCodeList) {

            List<CrossDirDataHistPOExt> filteredList = crossDirDataHistPOList.stream()
                    .filter(item -> crossId.equals(item.getCrossId()))
                    .filter(item -> 1 == item.getInOutType())
                    .filter(item -> dirCode.equals(item.getDirType()))
                    .collect(Collectors.toList());

            Integer maxDelayTime = filteredList.stream()
                    .map(CrossDirDataHistPOExt::getDelayTime)
                    .max(Integer::compareTo)
                    .orElse(0);
            Double maxQueueLength = filteredList.stream()
                    .mapToDouble(CrossDirDataHistPOExt::getQueueLength)
                    .max()
                    .orElse(0.0);


            int level = CrossUtil.calcEfficiencyLevel(maxDelayTime, maxQueueLength);
            if (level == 3) {
                res.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口延误较高效率较低");
            } else if (level == 4) {
                res.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口延误高效率低");
            }
        }
        if (CollectionUtil.isEmpty(res)) {
            res.add("该路口运行效率良好");
        }

        return res;
    }

    private List<MainlineCrossEvaluateVO.DirElement> buildDirElementList(
            String crossId, String metricName, List<BaseCrossDirInfoPO> baseCrossDirInfoPOList,
            List<CrossDirDataHistPOExt> crossDirDataHistPOList, List<PhaseEmptyResult> crossEmptyPhaseList) {

        // 获取路口所有方向
        List<BaseCrossDirInfoPO> collect = baseCrossDirInfoPOList.stream()
                .filter(item -> crossId.equals(item.getCrossId()))
                .collect(Collectors.toList());
        List<Integer> dirCodeList = collect.stream()
                .map(BaseCrossDirInfoPO::getDirType)
                .collect(Collectors.toList());

        List<MainlineCrossEvaluateVO.DirElement> res = new ArrayList<>();
        for (Integer dirCode : dirCodeList) {
            MainlineCrossEvaluateVO.DirElement dirElement = new MainlineCrossEvaluateVO.DirElement();
            dirElement.setDir(dirCode);

            List<CrossDirDataHistPOExt> filteredList = crossDirDataHistPOList.stream()
                    .filter(item -> crossId.equals(item.getCrossId()))
                    .filter(item -> 1 == item.getInOutType())
                    .filter(item -> dirCode.equals(item.getDirType()))
                    .collect(Collectors.toList());

            List<PhaseEmptyResult> dirPhaseEmptyList = crossEmptyPhaseList.stream()
                    .filter(p -> dirCode.equals(Integer.parseInt(p.getDirection())))
                    .collect(Collectors.toList());

            dirElement.setValue(calcDirValue(metricName, filteredList, dirPhaseEmptyList));

            res.add(dirElement);
        }
        return res;
    }

    private Integer calcDirValue(String metricName, List<CrossDirDataHistPOExt> crossDirDataHistPOList,
                                 List<PhaseEmptyResult> dirPhaseEmptyList) {
        int dirValue = 0;
        if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPOExt::getGreenLightEfficiency)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average * 100);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPOExt::getStopTimes)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPOExt::getDelayTime)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPOExt::getSpeed)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getDescription())) {
            double max = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPOExt::getQueueLength)
                    .max()
                    .orElse(0.0);
            dirValue = (int) Math.round(max);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPOExt::getEffusionRate)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average * 100);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.EMPTY_PHASE.getDescription())) {
            for (CrossDirDataHistPOExt po : crossDirDataHistPOList) {
                // 计算结束时间
                Date endTime = new Date(po.getStartTime().getTime() + 5 * 60 * 1000);  // 增加5分钟

                // 遍历每个 PhaseEmptyResult 来查找和累加 duration
                for (PhaseEmptyResult phaseEmptyResult : dirPhaseEmptyList) {
                    if (phaseEmptyResult.getDetectTime() * 1000 >= po.getStartTime().getTime()
                            && phaseEmptyResult.getDetectTime() * 1000 <= endTime.getTime()) {
                        // 累加 duration
                        po.setEmptyPhase(po.getEmptyPhase() + phaseEmptyResult.getDuration());
                    }
                }
            }
            double sum = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPOExt::getEmptyPhase)
                    .sum();
            dirValue = (int) Math.round(sum);
        }
        return dirValue;
    }

    private Integer calcCoordValue(
            List<Integer> coordDirCodeList, String crossId, List<CrossDirDataHistPOExt> crossDirDataHistPOList) {

        // 查询时段内路口协调方向数据
        List<CrossDirDataHistPOExt> filteredList = crossDirDataHistPOList.stream()
                .filter(item -> crossId.equals(item.getCrossId()))
                .filter(item -> 1 == item.getInOutType())
                .filter(item -> coordDirCodeList.contains(item.getDirType()))
                .collect(Collectors.toList());

        // 计算协调方向指标(不停车通过率)
        double coordValueDouble = (filteredList.stream()
                .mapToDouble(CrossDirDataHistPOExt::getNoStopRate)
                .average()
                .orElse(0.0) * 100);
        int coordValue = (int) Math.round(coordValueDouble);

        return coordValue;
    }

    private void setCurveElementListAndEvaluateList(
            String dirName, String metricName, Integer greenwaveId, Date boStartTime, Date boEndTime,
            String startHourMinuteStr, String endHourMinuteStr, MainlineSchemeEvaluateVO res, String strategyCode,
            Integer isCityBrain) {

        String boStartTimeStr = sdf.format(boStartTime);
        String boEndTimeStr = sdf.format(boEndTime);

        // 获取绿波历史数据
        List<GreenwaveHistPO> greenwaveHistPOList = greenwaveHistMapper
                .selectByIdAndTimeSection(greenwaveId, boStartTimeStr, boEndTimeStr);

        List<GreenwaveHistPO> filteredGreenwaveList;
        if (ObjectUtil.isEmpty(isCityBrain)) {
            // 过滤时段内的数据
            filteredGreenwaveList = greenwaveHistPOList.stream()
                    .filter(po -> {
                        try {
                            return isTimeInRange(po.getGmtModified(), startHourMinuteStr, endHourMinuteStr);
                        } catch (ParseException e) {
                            throw new RuntimeException(e);
                        }
                    })
                    .collect(Collectors.toList());
        } else {
            filteredGreenwaveList = greenwaveHistPOList.stream()
                    .filter(greenwaveHistPO -> !boStartTime.after(greenwaveHistPO.getGmtModified())
                            && !boEndTime.before(greenwaveHistPO.getGmtModified()))
                    .collect(Collectors.toList());
        }
        // 获取绿波指标
        double noparkPassRate = filteredGreenwaveList.stream()
                .mapToDouble(GreenwaveHistPO::getNoparkPassRate)
                .average()
                .orElse(0.0);
        double cordReliability = filteredGreenwaveList.stream()
                .mapToDouble(GreenwaveHistPO::getCordReliability)
                .average()
                .orElse(0.0);
        double uncoordinatePhaseQueue = filteredGreenwaveList.stream()
                .mapToDouble(GreenwaveHistPO::getUncoordinatePhaseQueue)
                .average()
                .orElse(0.0);
        double cordQueueRatio = filteredGreenwaveList.stream()
                .mapToDouble(GreenwaveHistPO::getCordQueueRatio)
                .average()
                .orElse(0.0);

        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        Map<String, GreenwaveCrossPO> uniqueCrossIdMap = greenwaveCrossPOList.stream()
                .collect(Collectors.toMap(
                        GreenwaveCrossPO::getCrossId,
                        Function.identity(),
                        (existing, replacement) -> existing
                ));
        List<GreenwaveCrossPO> filteredList = new ArrayList<>(uniqueCrossIdMap.values());

        // 获取各路口方向历史数据
        // 提取方向
        String[] split = dirName.split("向");
        String dirStr = split[0];
        Integer dir = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr);
        List<CrossDirDataHistPO> totalCrossDirDataHistPOList = new ArrayList<>();

        String boStartDayStr = dayFormat.format(boStartTime);
        String boEndDayStr = dayFormat.format(boEndTime);
        List<CrossDirDataHistPO> poInSectionList = crossDirDataHistMapper.selectByTimeSection(boStartDayStr, boEndDayStr, startHourMinuteStr, endHourMinuteStr);
        for (GreenwaveCrossPO greenwaveCrossPO : filteredList) {
            buildTotalCrossDirHistList(dirStr, dir, totalCrossDirDataHistPOList, greenwaveCrossPO, poInSectionList);
        }

        // 获取协调方向评价指标
        int coordSpeed = 0;
        int cordDelay = 0;
        int cordMaxQueue = 0;
        if (dirName.contains("非协调")) {
            String dirStr1 = split[1];
            String[] split1 = dirStr1.split("（");
            String s = split1[0];
            Integer dir1 = BaseEnum.SignalDirectionEnum.getCodeByName(s);
            List<CrossDirDataHistPO> totalCrossDirDataHistPOList1 = new ArrayList<>();
            for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOList) {
                buildTotalCrossDirHistList(dirStr1, dir1, totalCrossDirDataHistPOList1,
                        greenwaveCrossPO, poInSectionList);

                coordSpeed = (int) (totalCrossDirDataHistPOList1.stream()
                        .mapToDouble(CrossDirDataHistPO::getSpeed)
                        .average()
                        .orElse(0.0));
                cordDelay = (int) (totalCrossDirDataHistPOList1.stream()
                        .mapToDouble(CrossDirDataHistPO::getDelayTime)
                        .average()
                        .orElse(0.0));
                cordMaxQueue = (int) (totalCrossDirDataHistPOList1.stream()
                        .mapToDouble(CrossDirDataHistPO::getQueueLength)
                        .average()
                        .orElse(0.0));
            }
        } else {
            coordSpeed = (int) (totalCrossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getSpeed)
                    .average()
                    .orElse(0.0));
            cordDelay = (int) (totalCrossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getDelayTime)
                    .average()
                    .orElse(0.0));
            cordMaxQueue = (int) (totalCrossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getQueueLength)
                    .average()
                    .orElse(0.0));
        }

        // 生成评价结果
        List<String> evaluateList = new ArrayList<>();
        // 获取饱和度
        double saturation = totalCrossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getSturation)
                .average()
                .orElse(0.0);
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        Integer greenwaveDir = greenwaveInfoPO.getDir();
        if (greenwaveDir == 0 || greenwaveDir == 1) { // 单向绿波
            if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                if (noparkPassRate > 0.8 && cordReliability > 0.7) {
                    // 协调不停车通过率需大于80%，并且协调方案可靠性需大于70%
                    evaluateList.add("该方案符合控制策略");
                } else if (uncoordinatePhaseQueue < 0.5) {
                    // 如果以上不满足，则非协调相位二次排队需小于50%
                    evaluateList.add("该方案符合控制策略");
                } else {
                    evaluateList.add("该方案不符合控制策略");
                }
                evaluateList.add("协调不停车通过率" + noparkPassRate * 100 + StrategyAndMetricsEnum.Metrics.NO_PARK_PASS_RATE.getUnit());
                evaluateList.add("协调方案可靠性" + cordReliability * 100 + StrategyAndMetricsEnum.Metrics.CORD_RELIABILITY.getUnit());
                evaluateList.add("非协调方向二次排队" + uncoordinatePhaseQueue + StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getUnit());
            } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                if (cordQueueRatio < 0.6) {
                    // 协调路段排队空间占比需小于60%
                    evaluateList.add("该方案符合控制策略");
                } else if (uncoordinatePhaseQueue < 1.0) {
                    // 如果以上不满足，则非协调相位二次排队需小于100%
                    evaluateList.add("该方案符合控制策略");
                } else {
                    evaluateList.add("该方案不符合控制策略");
                }
                evaluateList.add("协调方向最大排队长度" + cordMaxQueue + StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getUnit());
                evaluateList.add("非协调方向二次排队" + uncoordinatePhaseQueue + StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getUnit());
            }
        } else { // 双向绿波
            if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                // 协调不停车通过率需大于50%，并且协调方案可靠性需大于70%
                if (noparkPassRate > 0.5 && cordReliability > 0.7) {
                    evaluateList.add("该方案符合控制策略");
                } else {
                    evaluateList.add("该方案不符合控制策略");
                }
                evaluateList.add("协调不停车通过率" + noparkPassRate * 100 + StrategyAndMetricsEnum.Metrics.NO_PARK_PASS_RATE.getUnit());
                evaluateList.add("协调方案可靠性" + cordReliability * 100 + StrategyAndMetricsEnum.Metrics.CORD_RELIABILITY.getUnit());
            } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                if (cordQueueRatio < 0.6) {
                    // 协调路段排队空间占比需小于60%
                    evaluateList.add("该方案符合控制策略");
                } else if (uncoordinatePhaseQueue < 1.0) {
                    // 如果以上不满足，则非协调相位二次排队需小于100%
                    evaluateList.add("该方案符合控制策略");
                } else {
                    evaluateList.add("该方案不符合控制策略");
                }
                evaluateList.add("协调方向最大排队长度" + cordMaxQueue + StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getUnit());
                evaluateList.add("非协调方向二次排队" + uncoordinatePhaseQueue + StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getUnit());
            }
        }
        evaluateList.add("协调方向平均速度" + coordSpeed + "km/h");
        evaluateList.add("协调方向平均延误" + cordDelay + "s");

        String s = evaluateList.get(0);
        if (s.contains("不符合")) {
            evaluateList.add("协调效果差");
        } else {
            evaluateList.add("协调效果良好");
        }
        res.setEvaluateList(evaluateList);

        // 生成曲线图
        List<String> timeArray;
        if (ObjectUtil.isEmpty(isCityBrain)) {
            timeArray = TimeArrayUtil.getCustomTimeIntervals(startHourMinuteStr, endHourMinuteStr, 5);
        } else {
            timeArray = TimeArrayUtil.getMultiDayTimeIntervals(boStartTime, boEndTime, 5);
        }
        if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.NO_PARK_PASS_RATE.getDescription())
                || Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.CORD_RELIABILITY.getDescription())
                || Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getDescription())
                || Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.CORD_QUEUE_RATIO.getDescription())) {

            // 按时间粒度聚合绿波历史数据
            Map<String, List<GreenwaveHistPO>> groupedByTime = greenwaveHistPOList.stream()
                    .collect(Collectors.groupingBy(po -> {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTime(po.getGmtModified());
                        return getHourMinuteString(calendar);
                    }));
            String enumName = StrategyAndMetricsEnum.Metrics.getEnumNameByDescription(metricName);
            evaluateMetrics(enumName, timeArray, groupedByTime, res);

        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.TRVAL_TIME.getDescription())) {
            // 按时间粒度聚合绿波历史数据
            Map<String, List<GreenwaveHistPO>> groupedByTime = greenwaveHistPOList.stream()
                    .collect(Collectors.groupingBy(po -> {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTime(po.getGmtModified());
                        return getHourMinuteString(calendar);
                    }));
            String enumName = StrategyAndMetricsEnum.Metrics.getEnumNameByDescription(metricName);
            evaluateMetricsNotRatio(enumName, timeArray, groupedByTime, res);
        } else {
            List<MainlineSchemeEvaluateVO.CurveElement> curveElementList = new ArrayList<>();
            // 按时间粒度聚合路口方向历史数据
            Map<String, List<CrossDirDataHistPO>> groupedByTime = totalCrossDirDataHistPOList.stream()
                    .collect(Collectors.groupingBy(po -> {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTimeInMillis((long) po.getBatchTime() * 1000);
                        return getHourMinuteString(calendar);
                    }));
            if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    OptionalDouble minAverage = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getGreenLightEfficiency)
                                    .average()
                                    .orElse(0.0))
                            .min();
                    int value = (int) (minAverage.orElse(0.0) * 100);

                    curveElement.setValue((double) value);
                    curveElementList.add(curveElement);
                }
            } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    double sumOfAverages = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getStopTimes)
                                    .average()
                                    .orElse(0.0))
                            .sum();

                    BigDecimal bd = new BigDecimal(sumOfAverages).setScale(2, RoundingMode.HALF_UP);
                    double rounded = bd.doubleValue();
                    curveElement.setValue(rounded);
                    curveElementList.add(curveElement);
                }
            } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    double sumOfAverages = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getDelayTime)
                                    .average()
                                    .orElse(0.0))
                            .sum();

                    BigDecimal bd = new BigDecimal(sumOfAverages).setScale(2, RoundingMode.HALF_UP);
                    double rounded = bd.doubleValue();
                    curveElement.setValue(rounded);
                    curveElementList.add(curveElement);
                }
            } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    double sumOfAverages = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getSpeed)
                                    .average()
                                    .orElse(0.0))
                            .average().orElse(0.0);

                    int value = (int) sumOfAverages;

                    curveElement.setValue((double)value);
                    curveElementList.add(curveElement);
                }
            } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    double sumOfMax = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getQueueLength)
                                    .max()
                                    .orElse(0.0))
                            .max().orElse(0.0);

                    int value = (int) sumOfMax;

                    curveElement.setValue((double)value);
                    curveElementList.add(curveElement);
                }
            } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.TRAFFIC_INDEX.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    double sumOfAverages = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getTrafficIndex)
                                    .average()
                                    .orElse(0.0))
                            .average().orElse(0.0);
                    BigDecimal bd = new BigDecimal(sumOfAverages).setScale(2, RoundingMode.HALF_UP);
                    double rounded = bd.doubleValue();
                    curveElement.setValue(rounded);
                    curveElementList.add(curveElement);
                }
            }
            res.setCurveElementList(curveElementList);
        }
    }

    private void evaluateMetricsNotRatio(
            String metricName, List<String> timeArray, Map<String, List<GreenwaveHistPO>> groupedByTime,
            MainlineSchemeEvaluateVO res) {

        List<MainlineSchemeEvaluateVO.CurveElement> curveElementList = new ArrayList<>();
        for (String hourMinute : timeArray) {
            MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
            curveElement.setMetricTime(hourMinute);
            List<GreenwaveHistPO> greenwaveHistPOListHourMinute = groupedByTime.get(hourMinute);

            ToDoubleFunction<GreenwaveHistPO> function;
            switch (metricName) {
                case "TRVAL_TIME":
                    function = GreenwaveHistPO::getTrvalTime;
                    break;
                default:
                    throw new IllegalArgumentException("Unknown metric: " + metricName);
            }
            if (CollectionUtil.isNotEmpty(greenwaveHistPOListHourMinute)) {
                int value = calculateAverageNotRatio(greenwaveHistPOListHourMinute, function);
                curveElement.setValue((double)value);
            } else {
                curveElement.setValue((double)0);
            }
            curveElementList.add(curveElement);
        }
        res.setCurveElementList(curveElementList);

    }

    private int calculateAverageNotRatio(
            List<GreenwaveHistPO> list, ToDoubleFunction<GreenwaveHistPO> function) {
        return (int) (list.stream().mapToDouble(function).average().orElse(0.0));
    }

    @NotNull
    private String getHourMinuteString(Calendar calendar) {
        int minute = calendar.get(Calendar.MINUTE);
        minute = (minute / 5) * 5;  // Round down to nearest 5 minutes
        calendar.set(Calendar.MINUTE, minute);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return hourMinuteFormat.format(calendar.getTime());
    }

    private void evaluateMetrics(String metricName, List<String> timeArray,
                                 Map<String, List<GreenwaveHistPO>> groupedByTime, MainlineSchemeEvaluateVO res) {
        List<MainlineSchemeEvaluateVO.CurveElement> curveElementList = new ArrayList<>();
        for (String hourMinute : timeArray) {
            MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
            curveElement.setMetricTime(hourMinute);
            List<GreenwaveHistPO> greenwaveHistPOListHourMinute = groupedByTime.get(hourMinute);

            ToDoubleFunction<GreenwaveHistPO> function;
            switch (metricName) {
                case "NO_PARK_PASS_RATE":
                    function = GreenwaveHistPO::getNoparkPassRate;
                    break;
                case "TRVAL_TIME":
                    function = GreenwaveHistPO::getTrvalTime;
                    break;
                case "CORD_RELIABILITY":
                    function = GreenwaveHistPO::getCordReliability;
                    break;
                case "UNCOORDINATE_PHASE_QUEUE":
                    function = GreenwaveHistPO::getUncoordinatePhaseQueue;
                    break;
                case "CORD_QUEUE_RATIO":
                    function = GreenwaveHistPO::getCordQueueRatio;
                    break;
                default:
                    throw new IllegalArgumentException("Unknown metric: " + metricName);
            }
            if (CollectionUtil.isNotEmpty(greenwaveHistPOListHourMinute)) {
                int value = calculateAverage(greenwaveHistPOListHourMinute, function);
                curveElement.setValue((double)value);
            } else {
                curveElement.setValue((double)0);
            }
            curveElementList.add(curveElement);
        }
        res.setCurveElementList(curveElementList);
    }

    private int calculateAverage(List<GreenwaveHistPO> list, ToDoubleFunction<GreenwaveHistPO> function) {
        return (int) (list.stream().mapToDouble(function).average().orElse(0.0) * 100);
    }

    private void buildTotalCrossDirHistList(
            String dirStr, Integer dir, List<CrossDirDataHistPO> totalCrossDirDataHistPOList,
            GreenwaveCrossPO greenwaveCrossPO, List<CrossDirDataHistPO> poInSectionList) {

        String crossId = greenwaveCrossPO.getCrossId();
        Integer dirCode = dir;
        dirCode = convertDirCode(dirStr, crossId, dirCode);
        Integer finalDirCode = dirCode;

        List<CrossDirDataHistPO> filteredList = poInSectionList.stream()
                .filter(po -> po.getCrossId().equals(crossId) && po.getDirType().equals(finalDirCode))
                .collect(Collectors.toList());

        totalCrossDirDataHistPOList.addAll(filteredList);
    }

    private Integer convertDirCode(String dirStr, String crossId, Integer dirCode) {
        if (Objects.equals(crossId, "13NF80B5QN0")) {
            // 霞景路方向转换
            if (Objects.equals(dirStr, "东")) {
                dirCode = 2; // 东北
            } else if (Objects.equals(dirStr, "西")) {
                dirCode = 6; // 西南
            }
        } else if (Objects.equals(crossId, "13NGH0B5RC0")) {
            // 洪山路方向转换
            if (Objects.equals(dirStr, "西")) {
                dirCode = 6; // 西南
            }
        }
        return dirCode;
    }


    private boolean isTimeInRange(Date date, String startStr, String endStr) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        Date startDate = sdf.parse(startStr);
        Date endDate = sdf.parse(endStr);

        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int hour = cal.get(Calendar.HOUR_OF_DAY);
        int minute = cal.get(Calendar.MINUTE);

        cal.setTime(startDate);
        int startHour = cal.get(Calendar.HOUR_OF_DAY);
        int startMinute = cal.get(Calendar.MINUTE);

        cal.setTime(endDate);
        int endHour = cal.get(Calendar.HOUR_OF_DAY);
        int endMinute = cal.get(Calendar.MINUTE);

        return (hour > startHour || (hour == startHour && minute >= startMinute))
                && (hour < endHour || (hour == endHour && minute <= endMinute));
    }

    private List<GreenwaveDetailVO.GreenwaveCross> buildGreenwaveCrossList(Integer greenwaveId) {
        List<GreenwaveDetailVO.GreenwaveCross> res = new ArrayList<>();
        List<GreenwaveCrossPO> greenwaveCrossList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        // 按唯一路口ID过滤
        Map<String, GreenwaveCrossPO> uniqueCrossIdMap = greenwaveCrossList.stream()
                .collect(Collectors.toMap(
                        GreenwaveCrossPO::getCrossId,
                        Function.identity(),
                        (existing, replacement) -> existing
                ));
        Collection<GreenwaveCrossPO> values = uniqueCrossIdMap.values();
        List<GreenwaveCrossPO> collect = values.stream()
                .sorted(Comparator.comparing(GreenwaveCrossPO::getSort))
                .collect(Collectors.toList());

        int size = collect.size();
        for (int i = 0; i < size; i++) {
            GreenwaveCrossPO greenwaveCrossPO = collect.get(i);
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = new GreenwaveDetailVO.GreenwaveCross();
            String crossId = greenwaveCrossPO.getCrossId();
            greenwaveCross.setCrossId(crossId);

            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            if (baseCrossInfoPO == null) {
                throw new RuntimeException("无此路口基础信息");
            }
            greenwaveCross.setCrossName(baseCrossInfoPO.getName());
            greenwaveCross.setIsKeyRoute(greenwaveCrossPO.getIsKeyRoute());
            greenwaveCross.setSort(greenwaveCrossPO.getSort());

            // 到下一个路口距离
            Integer distanceToNextCross = null;
            if (i < size - 1) { // 不是最后一个路口
                double nextCrossLen = greenwaveCrossPO.getNextCrossLen();
                distanceToNextCross = (int) nextCrossLen;
            }
            greenwaveCross.setDistanceToNextCross(distanceToNextCross);

            res.add(greenwaveCross);
        }
        return res;
    }

    private List<MainlineSchemeAnalysisVO.GreenwaveData> buildGreenwaveData(
            Map<String, LineSchemeDTO> lineSchemeBuffer) {

        List<MainlineSchemeAnalysisVO.GreenwaveData> res = new ArrayList<>();

        for (Map.Entry<String, LineSchemeDTO> entry : lineSchemeBuffer.entrySet()) {
            MainlineSchemeAnalysisVO.GreenwaveData greenwaveData = new MainlineSchemeAnalysisVO.GreenwaveData();
            String lineSchemeName = entry.getKey();
            greenwaveData.setName(lineSchemeName);
            LineSchemeDTO value = entry.getValue();
            Integer greenwaveId = value.getGreenwaveId();
            greenwaveData.setGreenwaveId(greenwaveId);
            greenwaveData.setDirList(buildDirList(lineSchemeName));

            res.add(greenwaveData);
        }

        return res;
    }

    private List<MainlineSchemeAnalysisVO.DirectionList> buildDirList(String lineSchemeName) {
        String[] split = lineSchemeName.split(" "); // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 19:00-20:00
        String greenwaveName = split[0] + " " + split[1] + " " + split[2];

        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectByName(greenwaveName);
        Integer greenwaveId = greenwaveInfoPO.getId();
        List<MainlineSchemeAnalysisVO.DirectionList> res = new ArrayList<>();

        String dirName = split[1];
        Integer coordinateDir = greenwaveInfoPO.getDir();

        if (coordinateDir == 0) { // 正向
            MainlineSchemeAnalysisVO.DirectionList directionList = new MainlineSchemeAnalysisVO.DirectionList();
            directionList.setName(dirName + "（协调方向）");
            directionList.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList);
            MainlineSchemeAnalysisVO.DirectionList directionList1 = new MainlineSchemeAnalysisVO.DirectionList();
            String dirNameRev = reverseDirName(dirName, "向");
            directionList1.setName(dirNameRev + "（非协调方向）");
            directionList1.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList1);
        } else if (coordinateDir == 1) { // 反向
            MainlineSchemeAnalysisVO.DirectionList directionList = new MainlineSchemeAnalysisVO.DirectionList();
            directionList.setName(dirName + "（非协调方向）");
            directionList.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList);
            MainlineSchemeAnalysisVO.DirectionList directionList1 = new MainlineSchemeAnalysisVO.DirectionList();
            String dirNameRev = reverseDirName(dirName, "向");
            directionList1.setName(dirNameRev + "（协调方向）");
            directionList1.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList1);
        } else { // 双向
            MainlineSchemeAnalysisVO.DirectionList directionList = new MainlineSchemeAnalysisVO.DirectionList();
            directionList.setName(dirName + "（协调方向）");
            directionList.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList);
            MainlineSchemeAnalysisVO.DirectionList directionList1 = new MainlineSchemeAnalysisVO.DirectionList();
            String dirNameRev = reverseDirName(dirName, "向");
            directionList1.setName(dirNameRev + "（协调方向）");
            directionList1.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList1);
        }

        return res;
    }

    private List<MainlineSchemeAnalysisVO.Metric> buildMetricList(Integer greenwaveId, Integer coordinateDir) {
        List<MainlineSchemeAnalysisVO.Metric> res = new ArrayList<>();
        // 确定控制策略
        StrategyPO strategyPO = getStrategyPO(greenwaveId);
        String strategyCode = strategyPO.getStrategyCode();

        if (coordinateDir == 0 || coordinateDir == 1) { // 单向绿波
            if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                addMetrics(res);
                MainlineSchemeAnalysisVO.Metric metric = new MainlineSchemeAnalysisVO.Metric();
                metric.setMetricName(StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getDescription());
                metric.setIsShownRight(0);
                res.add(metric);
            } else {
                fillBalanceMetrics(res, strategyCode);
            }
        } else { // 双向绿波
            if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                addMetrics(res);
            } else fillBalanceMetrics(res, strategyCode);
        }

        return res;
    }

    private void fillBalanceMetrics(List<MainlineSchemeAnalysisVO.Metric> res, String strategyCode) {
        if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
            MainlineSchemeAnalysisVO.Metric metric2 = new MainlineSchemeAnalysisVO.Metric();
            metric2.setMetricName(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getDescription());
            metric2.setIsShownRight(1);
            res.add(metric2);

            MainlineSchemeAnalysisVO.Metric metric = new MainlineSchemeAnalysisVO.Metric();
            metric.setMetricName(StrategyAndMetricsEnum.Metrics.CORD_QUEUE_RATIO.getDescription());
            metric.setIsShownRight(0);
            res.add(metric);

            MainlineSchemeAnalysisVO.Metric metric1 = new MainlineSchemeAnalysisVO.Metric();
            metric1.setMetricName(StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getDescription());
            metric1.setIsShownRight(0);
            res.add(metric1);
        }
    }

    private void addMetrics(List<MainlineSchemeAnalysisVO.Metric> res) {
        MainlineSchemeAnalysisVO.Metric metric = new MainlineSchemeAnalysisVO.Metric();
        metric.setMetricName(StrategyAndMetricsEnum.Metrics.NO_PARK_PASS_RATE.getDescription());
        metric.setIsShownRight(0);
        res.add(metric);

        MainlineSchemeAnalysisVO.Metric metric2 = new MainlineSchemeAnalysisVO.Metric();
        metric2.setMetricName(StrategyAndMetricsEnum.Metrics.TRVAL_TIME.getDescription());
        metric2.setIsShownRight(0);
        res.add(metric2);

        MainlineSchemeAnalysisVO.Metric metric8 = new MainlineSchemeAnalysisVO.Metric();
        metric8.setMetricName(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getDescription());
        metric8.setIsShownRight(1);
        res.add(metric8);

        MainlineSchemeAnalysisVO.Metric metric3 = new MainlineSchemeAnalysisVO.Metric();
        metric3.setMetricName(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getDescription());
        metric3.setIsShownRight(1);
        res.add(metric3);

        MainlineSchemeAnalysisVO.Metric metric4 = new MainlineSchemeAnalysisVO.Metric();
        metric4.setMetricName(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getDescription());
        metric4.setIsShownRight(1);
        res.add(metric4);

        MainlineSchemeAnalysisVO.Metric metric5 = new MainlineSchemeAnalysisVO.Metric();
        metric5.setMetricName(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getDescription());
        metric5.setIsShownRight(1);
        res.add(metric5);

        MainlineSchemeAnalysisVO.Metric metric6 = new MainlineSchemeAnalysisVO.Metric();
        metric6.setMetricName(StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getDescription());
        metric6.setIsShownRight(1);
        res.add(metric6);

        MainlineSchemeAnalysisVO.Metric metric7 = new MainlineSchemeAnalysisVO.Metric();
        metric7.setMetricName(StrategyAndMetricsEnum.Metrics.CORD_RELIABILITY.getDescription());
        metric7.setIsShownRight(0);
        res.add(metric7);

        MainlineSchemeAnalysisVO.Metric metric9 = new MainlineSchemeAnalysisVO.Metric();
        metric9.setMetricName(StrategyAndMetricsEnum.Metrics.TRAFFIC_INDEX.getDescription());
        metric9.setIsShownRight(0);
        res.add(metric9);

        MainlineSchemeAnalysisVO.Metric metric10 = new MainlineSchemeAnalysisVO.Metric();
        metric10.setMetricName(StrategyAndMetricsEnum.Metrics.EMPTY_PHASE.getDescription());
        metric10.setIsShownRight(1);
        metric10.setIsShownLeft(0);
        res.add(metric10);

        MainlineSchemeAnalysisVO.Metric metric11 = new MainlineSchemeAnalysisVO.Metric();
        metric11.setMetricName(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getDescription());
        metric11.setIsShownRight(1);
        metric11.setIsShownLeft(0);
        res.add(metric11);
    }

    private String reverseDirName(String input, String character) {
        String[] split = input.split(character);
        return split[1] + character + split[0];
    }

    private List<String> calcEvaluateData(
            List<GreenwaveHistPO> filteredList, Map<String, LineSchemeDTO> lineSchemeBuffer, int boStartTimeStamp,
            int boEndTimeStamp, Date boStartTime, Date boEndTime) {

        Set<String> qualifiedSet = new HashSet<>();
        Set<String> notQualifiedSet = new HashSet<>();

        for (String greenwaveName : lineSchemeBuffer.keySet()) {
            String[] split = greenwaveName.split(" "); // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 19:00-20:00
            String baseGreenwaveName = split[0] + " " + split[1] + " " + split[2]; // 旅游路 东向西 转山西路至霞景路路段:正向绿波
            String timeSpan = split[4]; // 19:00-20:00
            String[] split1 = timeSpan.split("-");
            String startHourMinuteStr = split1[0];
            String endHourMinuteStr = split1[1];

            List<GreenwaveHistPO> filteredList1 = filteredList.stream()
                    .filter(po -> baseGreenwaveName.equals(po.getGreenwaveName()))
                    .collect(Collectors.toList());

            // 过滤时段内的数据
            List<GreenwaveHistPO> filteredGreenwaveList = filteredList1.stream()
                    .filter(po -> {
                        try {
                            return isTimeInRange(po.getGmtModified(), startHourMinuteStr, endHourMinuteStr);
                        } catch (ParseException e) {
                            throw new RuntimeException(e);
                        }
                    })
                    .collect(Collectors.toList());

            // 获取绿波指标
            double noparkPassRate = filteredGreenwaveList.stream()
                    .mapToDouble(GreenwaveHistPO::getNoparkPassRate)
                    .average()
                    .orElse(0.0);
            double cordReliability = filteredGreenwaveList.stream()
                    .mapToDouble(GreenwaveHistPO::getCordReliability)
                    .average()
                    .orElse(0.0);
            double uncoordinatePhaseQueue = filteredGreenwaveList.stream()
                    .mapToDouble(GreenwaveHistPO::getUncoordinatePhaseQueue)
                    .average()
                    .orElse(0.0);
            double cordQueueRatio = filteredGreenwaveList.stream()
                    .mapToDouble(GreenwaveHistPO::getCordQueueRatio)
                    .average()
                    .orElse(0.0);

            GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectByName(baseGreenwaveName);
            Integer greenwaveId = greenwaveInfoPO.getId();
            List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
            Map<String, GreenwaveCrossPO> uniqueCrossIdMap = greenwaveCrossPOList.stream()
                    .collect(Collectors.toMap(
                            GreenwaveCrossPO::getCrossId,
                            Function.identity(),
                            (existing, replacement) -> existing
                    ));
            List<GreenwaveCrossPO> crossList = new ArrayList<>(uniqueCrossIdMap.values());

            // 获取各路口方向历史数据
            // 提取方向
            String s = split[1]; // 东向西
            String[] split2 = s.split("向");
            String dirStr = split2[0];
            Integer dir = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr);
            List<CrossDirDataHistPO> totalCrossDirDataHistPOList = new ArrayList<>();

            String boStartDayStr = dayFormat.format(boStartTime);
            String boEndDayStr = dayFormat.format(boEndTime);
            List<CrossDirDataHistPO> poInSectionList = crossDirDataHistMapper.selectByTimeSection(boStartDayStr, boEndDayStr, startHourMinuteStr, endHourMinuteStr);

            for (GreenwaveCrossPO greenwaveCrossPO : crossList) {
                buildTotalCrossDirHistList(dirStr, dir, totalCrossDirDataHistPOList, greenwaveCrossPO, poInSectionList);
            }

            // 获取饱和度
            double saturation = totalCrossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getSturation)
                    .average()
                    .orElse(0.0);

            Integer greenwaveDir = greenwaveInfoPO.getDir();
            // 确定控制策略
            StrategyPO strategyPO = getStrategyPO(greenwaveId);
            String strategyCode = strategyPO.getStrategyCode();
            if (greenwaveDir == 0 || greenwaveDir == 1) { // 单向绿波
                if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                    if (saturation < 0.8) {
                        if (noparkPassRate > 0.8 && cordReliability > 0.7) {
                            // 协调不停车通过率需大于80%，并且协调方案可靠性需大于70%
                            qualifiedSet.add(greenwaveName);
                        } else if (uncoordinatePhaseQueue < 0.5) {
                            // 如果以上不满足，则非协调相位二次排队需小于50%
                            qualifiedSet.add(greenwaveName);
                        } else {
                            notQualifiedSet.add(greenwaveName);
                        }
                    } else {
                        // 协调方案可靠性需大于70%
                        if (cordReliability > 0.7) {
                            qualifiedSet.add(greenwaveName);
                        } else {
                            notQualifiedSet.add(greenwaveName);
                        }
                    }
                } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                    if (cordQueueRatio < 0.6) {
                        // 协调路段排队空间占比需小于60%
                        qualifiedSet.add(greenwaveName);
                    } else if (uncoordinatePhaseQueue < 1.0) {
                        // 如果以上不满足，则非协调相位二次排队需小于100%
                        qualifiedSet.add(greenwaveName);
                    } else {
                        notQualifiedSet.add(greenwaveName);
                    }
                }
            } else { // 双向绿波
                if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                    // 协调不停车通过率需大于50%，并且协调方案可靠性需大于70%
                    if (noparkPassRate > 0.5 && cordReliability > 0.7) {
                        qualifiedSet.add(greenwaveName);
                    } else {
                        notQualifiedSet.add(greenwaveName);
                    }
                } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                    if (cordQueueRatio < 0.6) {
                        // 协调路段排队空间占比需小于60%
                        qualifiedSet.add(greenwaveName);
                    } else if (uncoordinatePhaseQueue < 1.0) {
                        // 如果以上不满足，则非协调相位二次排队需小于100%
                        qualifiedSet.add(greenwaveName);
                    } else {
                        notQualifiedSet.add(greenwaveName);
                    }
                }
            }
        }
        List<String> res = new ArrayList<>();
        qualifiedSet.removeAll(notQualifiedSet); // 只要有一次不符合，则剔除
        res.add("符合控制策略方案数 " + qualifiedSet.size() + "个");
        res.add("不符合控制策略方案数 " + notQualifiedSet.size() + "个");
        return res;
    }

    private String findKey(Map<String, LineSchemeDTO> lineSchemeBuffer, Integer greenwaveId, Date gmtModified) {
        for (Map.Entry<String, LineSchemeDTO> entry : lineSchemeBuffer.entrySet()) {
            String key = entry.getKey();
            LineSchemeDTO lineSchemeDTO = entry.getValue();

            if (lineSchemeDTO != null) {
                List<GreenwaveHistPO> greenwaveHistPOList = lineSchemeDTO.getGreenwaveHistPOList();
                if (greenwaveHistPOList != null) {
                    for (GreenwaveHistPO greenwaveHistPO : greenwaveHistPOList) {
                        if (greenwaveHistPO != null &&
                                Objects.equals(greenwaveHistPO.getGreenId(), greenwaveId) &&
                                Objects.equals(greenwaveHistPO.getGmtModified(), gmtModified)) {
                            return key;
                        }
                    }
                }
            }
        }
        return null;
    }

    private StrategyPO getStrategyPO(Integer greenwaveId) {
        GreenwaveScenePO greenwaveScenePO = greenwaveSceneMapper.selectByGreenwaveId(greenwaveId);
        Integer sceneId = greenwaveScenePO.getSceneId();
        List<SceneStrategyPO> sceneStrategyPOList = sceneStrategyMapper.selectBySceneId(sceneId);
        Optional<SceneStrategyPO> maxPriorityPO = sceneStrategyPOList.stream()
                .min(Comparator.comparingInt(SceneStrategyPO::getSceneStrategyPriority));
        SceneStrategyPO sceneStrategyPO = maxPriorityPO.get();
        Integer strategyId = sceneStrategyPO.getStrategyId();
        StrategyPO strategyPO = strategyMapper.selectById(strategyId);
        return strategyPO;
    }

    private List<BaseCrossInfoPO> getGreenwaveCross(
            Integer greenwaveId, List<GreenwaveCrossPO> baseGreenwaveCrossPOList,
            List<BaseCrossInfoPO> baseCrossInfoPOList) {

        List<BaseCrossInfoPO> res = new ArrayList<>();
        List<GreenwaveCrossPO> greenwaveCrosses = baseGreenwaveCrossPOList.stream()
                .filter(po -> Objects.equals(po.getGreenId(), greenwaveId))
                .collect(Collectors.toList());
        for (GreenwaveCrossPO greenwaveCross : greenwaveCrosses) {
            String crossId = greenwaveCross.getCrossId();
            Optional<BaseCrossInfoPO> first = baseCrossInfoPOList.stream()
                    .filter(po -> Objects.equals(po.getId(), crossId))
                    .findFirst();
            BaseCrossInfoPO baseCrossInfoPO = first.get();
            res.add(baseCrossInfoPO);
        }
        return res;
    }

    private Date combineDateAndTime(Date day, String hourMinuteStr) {
        String[] hourMinute = hourMinuteStr.split(":");
        int hour = Integer.parseInt(hourMinute[0]);
        int minute = Integer.parseInt(hourMinute[1]);

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(day);

        calendar.set(Calendar.HOUR_OF_DAY, hour);
        calendar.set(Calendar.MINUTE, minute);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        return calendar.getTime();
    }

    private List<MainlineSchemeAnalysisVO.CrossData> calcCrossData(Map<String, LineSchemeDTO> lineSchemeBuffer)
            throws ParseException {

        List<MainlineSchemeAnalysisVO.CrossData> res = new ArrayList<>();

        Set<String> greenwaveSchemes = lineSchemeBuffer.keySet();

        Map<String, Integer> crossTimesMap = new HashMap<>();
        // 找出绿波对应的关键路口
        List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();
        List<GreenwaveCrossPO> baseGreenwaveCrossPOList = greenwaveCrossMapper.selectByKeyRoute();
        List<CrossSectionPO> baseCrossSectionPOList = baseCrossSectionMapper.selectAll();
        List<BaseCrossPlanPO> baseCrossPlanPOList = baseCrossPlanMapper.selectAll();

        for (String greenwaveScheme : greenwaveSchemes) {
            String[] split = greenwaveScheme.split(" "); // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 07:00-09:00
            String greenwaveName = String.join(" ", Arrays.copyOfRange(split, 0, 3));// 旅游路 东向西 转山西路至霞景路路段:正向绿波
            Optional<GreenwaveInfoPO> result = greenwaveInfoPOList.stream()
                    .filter(greenwaveInfo -> greenwaveName.equals(greenwaveInfo.getName()))
                    .findFirst();
            Integer greenwaveId = result.get().getId();

            // 所有的关键路口
            List<GreenwaveCrossPO> greenwaveCrossPOList = baseGreenwaveCrossPOList.stream()
                    .filter(po -> Objects.equals(po.getGreenId(), greenwaveId))
                    .collect(Collectors.toList());

            for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOList) {
                Integer sectionId = greenwaveCrossPO.getSectionId();
                String crossId = greenwaveCrossPO.getCrossId();
                // 判断绿波是否在时段内
                boolean greenwaveInSection = isGreenwaveSchemeInSection(
                        split, sectionId, baseCrossSectionPOList, baseCrossPlanPOList);

                if (greenwaveInSection) {
                    crossTimesMap.put(crossId, crossTimesMap.getOrDefault(crossId, 0) + 1);
                    break;
                }
            }

        }

        // 统计关键路口次数
        for (Map.Entry<String, Integer> entry : crossTimesMap.entrySet()) {
            MainlineSchemeAnalysisVO.CrossData crossData = new MainlineSchemeAnalysisVO.CrossData();
            String crossId = entry.getKey();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            String crossName = baseCrossInfoPO.getName();
            crossData.setName(crossName);
            crossData.setTimes(entry.getValue());
            res.add(crossData);
        }

        return res;
    }

    private boolean isGreenwaveSchemeInSection(
            String[] split, Integer sectionId, List<CrossSectionPO> baseCrossSectionPOList,
            List<BaseCrossPlanPO> baseCrossPlanPOList) {

        String planName = split[3]; // 工作日
        String startEndTime = split[4]; // 07:00-09:00
        String[] split1 = startEndTime.split("-");
        String startTime = split1[0];

        Optional<CrossSectionPO> optCrossSectionPO = baseCrossSectionPOList.stream()
                .filter(po -> Objects.equals(po.getId(), sectionId))
                .findFirst();
        CrossSectionPO crossSectionPO = optCrossSectionPO.get();

        Integer planId = crossSectionPO.getPlanId();
        Optional<BaseCrossPlanPO> optBaseCrossPlanPO = baseCrossPlanPOList.stream()
                .filter(po -> Objects.equals(po.getId(), planId))
                .findFirst();
        BaseCrossPlanPO baseCrossPlanPO = optBaseCrossPlanPO.get();

        String name = baseCrossPlanPO.getName();
        if (Objects.equals(name, planName)) {
            String poStartTime = crossSectionPO.getStartTime();
            if (Objects.equals(poStartTime, startTime)) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    private GreenwaveInSectionDTO isGreenwaveInSection(
            Date histGreenwaveGmtModified, Integer sectionId, List<CrossSectionPO> baseCrossSectionPOList,
            List<BaseCrossPlanPO> baseCrossPlanPOList)
            throws ParseException {

        GreenwaveInSectionDTO res = new GreenwaveInSectionDTO();

        Optional<CrossSectionPO> optCrossSectionPO = baseCrossSectionPOList.stream()
                .filter(po -> Objects.equals(po.getId(), sectionId))
                .findFirst();
        CrossSectionPO crossSectionPO = optCrossSectionPO.get();

        Integer planId = crossSectionPO.getPlanId();
        Optional<BaseCrossPlanPO> optBaseCrossPlanPO = baseCrossPlanPOList.stream()
                .filter(po -> Objects.equals(po.getId(), planId))
                .findFirst();
        BaseCrossPlanPO baseCrossPlanPO = optBaseCrossPlanPO.get();


        String name = baseCrossPlanPO.getName();
        if (!name.contains("工作日") && !name.contains("周末")) {
            // 特殊日期
            Date nameDate = chineseDayFormat.parse(name);

            Calendar calendar1 = Calendar.getInstance();
            calendar1.setTime(histGreenwaveGmtModified);
            int month1 = calendar1.get(Calendar.MONTH) + 1;  // Calendar.MONTH is 0-based
            int day1 = calendar1.get(Calendar.DAY_OF_MONTH);

            Calendar calendar2 = Calendar.getInstance();
            calendar2.setTime(nameDate);
            int month2 = calendar2.get(Calendar.MONTH) + 1;  // Calendar.MONTH is 0-based
            int day2 = calendar2.get(Calendar.DAY_OF_MONTH);

            if (month1 == month2 && day1 == day2) {
                res.setGreenwaveInSection(isGreenwaveHourMinuteInSection(histGreenwaveGmtModified, crossSectionPO, res));
                res.setPlanName(name);
                res.setPlanId(planId);
            } else {
                res.setGreenwaveInSection(false);
            }

        } else if (name.contains("工作日")) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(histGreenwaveGmtModified);
            int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
            if (dayOfWeek >= Calendar.MONDAY && dayOfWeek <= Calendar.FRIDAY) {
                // 判断小时分钟是否交叉
                res.setGreenwaveInSection(isGreenwaveHourMinuteInSection(histGreenwaveGmtModified, crossSectionPO, res));
                res.setPlanName(name);
                res.setPlanId(planId);
            } else {
                res.setGreenwaveInSection(false);
            }
        } else if (name.contains("周末")) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(histGreenwaveGmtModified);
            int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
            if (dayOfWeek == Calendar.SATURDAY || dayOfWeek == Calendar.SUNDAY) {
                // 判断小时分钟是否交叉
                res.setGreenwaveInSection(isGreenwaveHourMinuteInSection(histGreenwaveGmtModified, crossSectionPO, res));
                res.setPlanName(name);
                res.setPlanId(planId);
            } else {
                res.setGreenwaveInSection(false);
            }
        }
        return res;
    }

    private boolean isGreenwaveHourMinuteInSection
            (Date histGreenwaveGmtModified, CrossSectionPO crossSectionPO, GreenwaveInSectionDTO res)
            throws ParseException {

        String startTime = crossSectionPO.getStartTime();
        String endTime = crossSectionPO.getEndTime();

        Date startDate = hourMinuteFormat.parse(startTime);
        Date endDate = hourMinuteFormat.parse(endTime);

        LocalDateTime localDateTime = histGreenwaveGmtModified.toInstant()
                .atZone(ZoneId.systemDefault())
                .toLocalDateTime();
        String histTimeStr = localDateTime.format(safeHourMinuteFormat);
        Date histTime = hourMinuteFormat.parse(histTimeStr);

        res.setStartTime(startTime);
        res.setEndTime(endTime);

        return (histTime.equals(startDate) || histTime.after(startDate)) && histTime.before(endDate);
    }

    private List<MainlineSchemeAnalysisVO.DirectionData> calcSceneData(Map<String, LineSchemeDTO> lineSchemeBuffer) {
        List<MainlineSchemeAnalysisVO.DirectionData> res = new ArrayList<>();

        Set<String> greenwaveSchemes = lineSchemeBuffer.keySet();

        HashMap<String, Integer> map = new HashMap<>();
        for (String greenwaveScheme : greenwaveSchemes) {
            String[] split = greenwaveScheme.split(":"); // 旅游路东向西转山西路至霞景路路段:正向绿波 工作日 23:00-24:00
            String firstChar = split[1].substring(0, 1);
            if (firstChar.equals("正")) { // 正向
                map.put("单向绿波上行", map.getOrDefault("单向绿波上行", 0) + 1);
            } else if (firstChar.equals("反")) { // 反向
                map.put("单向绿波下行", map.getOrDefault("单向绿波下行", 0) + 1);
            } else { // 双向
                map.put("双向绿波", map.getOrDefault("双向绿波", 0) + 1);
            }
        }

        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            MainlineSchemeAnalysisVO.DirectionData directionData = new MainlineSchemeAnalysisVO.DirectionData();
            directionData.setName(entry.getKey());
            directionData.setTimes(entry.getValue());
            res.add(directionData);
        }

        return res;
    }

    private Date calcDate(String timeStr, Date startTime) throws ParseException {
        // 使用 Calendar 来获取年、月、日信息
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startTime);
        // 使用 SimpleDateFormat 来获取时和分
        Date time = hourMinuteFormat.parse(timeStr);
        Calendar timeCal = Calendar.getInstance();
        timeCal.setTime(time);
        // 组合两者
        calendar.set(Calendar.HOUR_OF_DAY, timeCal.get(Calendar.HOUR_OF_DAY));
        calendar.set(Calendar.MINUTE, timeCal.get(Calendar.MINUTE));
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        // 转换回 Date 对象
        Date startDatetime = calendar.getTime();
        return startDatetime;
    }

    private List<String> buildTimeList(Date startTime, Date endTime, Integer minutes) {
        List<String> timeList = new ArrayList<>();
        Calendar cal = Calendar.getInstance();
        cal.setTime(startTime);

        long timeDifference = endTime.getTime() - startTime.getTime();
        boolean exceeds24Hours = timeDifference > (24 * 60 * 60 * 1000);

        Set<String> timeSet = new HashSet<>();

        while (cal.getTime().before(endTime) || cal.getTime().equals(endTime)) {
            String formattedTime = hourMinuteFormat.format(cal.getTime());

            if (exceeds24Hours) {
                if (timeSet.contains(formattedTime)) {
                    // 如果 startTime 和 endTime 时间差大于24小时，则 timeList 只保留第一条重复元素之前的元素
                    break;
                }
                timeSet.add(formattedTime);
            }
            timeList.add(formattedTime);
            cal.add(Calendar.MINUTE, minutes);
        }
        return timeList;
    }

    public static String formatTime(int totalMinutes) {
        int hours = totalMinutes / 60;
        int minutes = totalMinutes % 60;
        return String.format("%02d:%02d", hours, minutes);
    }

    private List<String> buildLaneMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> dirList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO dirPo : dirList) {
            Integer dirType = dirPo.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);

            List<LaneInfoPO> laneList = laneInfoMapper.selectInDirByCrossIdAndDir(crossId, dirType);
            for (LaneInfoPO lanePO : laneList) {
                Integer turn = lanePO.getTurn();
                String turnDesc = TurnConvertEnum.getDescByKey(turn);
                Integer sort = lanePO.getSort();
                res.add(dirName + "进口" + turnDesc + sort + "车道");
            }
        }
        return res;
    }

    private List<String> buildTurnMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> dirList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO dirPo : dirList) {
            Integer dirType = dirPo.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);

            List<CrossTurnInfoPO> turnList = baseCrossTurnInfoMapper.selectByCrossIdAndDir(crossId, dirType);
            for (CrossTurnInfoPO turnPO : turnList) {
                String turnType = turnPO.getTurnType();
                String turnDesc = TurnConvertEnum.getDescByCode(turnType);
                res.add(dirName + "进口" + turnDesc);
            }
        }
        return res;
    }

    private List<String> buildDirMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> filteredList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO po : filteredList) {
            Integer dirType = po.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
            res.add(dirName + "进口");
        }
        return res;
    }

}
